@@ -3,12 +3,13 @@ from logging import getLogger
 
 from cached_property import threaded_cached_property
 
-from .autodiscover import Autodiscovery
+from .autodiscover.discovery.pox import PoxAutodiscovery
+from .autodiscover.discovery.soap import SoapAutodiscovery
 from .configuration import Configuration
 from .credentials import ACCESS_TYPES, DELEGATE, IMPERSONATION
 from .errors import InvalidEnumValue, InvalidTypeError, UnknownTimeZone
 from .ewsdatetime import UTC, EWSTimeZone
-from .fields import FieldPath
+from .fields import FieldPath, TextField
 from .folders import (
     AdminAuditLogs,
     ArchiveDeletedItems,
@@ -55,7 +56,7 @@ from .folders import (
     VoiceMail,
 )
 from .items import ALL_OCCURRENCES, AUTO_RESOLVE, HARD_DELETE, ID_ONLY, SAVE_ONLY, SEND_TO_NONE
-from .properties import Mailbox, SendingAs
+from .properties import EWSElement, Mailbox, SendingAs
 from .protocol import Protocol
 from .queryset import QuerySet
 from .services import (
@@ -81,36 +82,24 @@ from .util import get_domain, peek
 log = getLogger(__name__)
 
 
-class Identity:
+class Identity(EWSElement):
     """Contains information that uniquely identifies an account. Currently only used for SOAP impersonation headers."""
 
-    def __init__(self, primary_smtp_address=None, smtp_address=None, upn=None, sid=None):
-        """
+    ELEMENT_NAME = "ConnectingSID"
 
-        :param primary_smtp_address: The primary email address associated with the account (Default value = None)
-        :param smtp_address: The (non-)primary email address associated with the account (Default value = None)
-        :param upn: (Default value = None)
-        :param sid: (Default value = None)
-        :return:
-        """
-        self.primary_smtp_address = primary_smtp_address
-        self.smtp_address = smtp_address
-        self.upn = upn
-        self.sid = sid
-
-    def __eq__(self, other):
-        return all(getattr(self, k) == getattr(other, k) for k in self.__dict__)
-
-    def __hash__(self):
-        return hash(repr(self))
-
-    def __repr__(self):
-        return self.__class__.__name__ + repr((self.primary_smtp_address, self.smtp_address, self.upn, self.sid))
+    # We have multiple options for uniquely identifying the user. Here's a prioritized list in accordance with
+    # https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/connectingsid
+    sid = TextField(field_uri="SID")
+    upn = TextField(field_uri="PrincipalName")
+    smtp_address = TextField(field_uri="SmtpAddress")  # The (non-)primary email address for the account
+    primary_smtp_address = TextField(field_uri="PrimarySmtpAddress")  # The primary email address for the account
 
 
 class Account:
     """Models an Exchange server user account."""
 
+    DEFAULT_DISCOVERY_CLS = PoxAutodiscovery
+
     def __init__(
         self,
         primary_smtp_address,
@@ -129,7 +118,7 @@ class Account:
         :param access_type: The access type granted to 'credentials' for this account. Valid options are 'delegate'
             and 'impersonation'. 'delegate' is default if 'credentials' is set. Otherwise, 'impersonation' is default.
         :param autodiscover: Whether to look up the EWS endpoint automatically using the autodiscover protocol.
-            (Default value = False)
+            Can also be set to "pox" or "soap" to choose the autodiscover implementation (Default value = False)
         :param credentials: A Credentials object containing valid credentials for this account. (Default value = None)
         :param config: A Configuration object containing EWS endpoint information. Required if autodiscover is disabled
             (Default value = None)
@@ -176,7 +165,12 @@ class Account:
                     credentials = config.credentials
             else:
                 auth_type, retry_policy, version = None, None, None
-            self.ad_response, self.protocol = Autodiscovery(
+            discovery_cls = {
+                "pox": PoxAutodiscovery,
+                "soap": SoapAutodiscovery,
+                True: self.DEFAULT_DISCOVERY_CLS,
+            }[autodiscover]
+            self.ad_response, self.protocol = discovery_cls(
                 email=primary_smtp_address, credentials=credentials
             ).discover()
             # Let's not use the auth_package hint from the AD response. It could be incorrect and we can just guess.
