@@ -1,5 +1,6 @@
 from .cache import AutodiscoverCache, autodiscover_cache
-from .discovery import Autodiscovery, discover
+from .discovery.pox import PoxAutodiscovery as Autodiscovery
+from .discovery.pox import discover
 from .protocol import AutodiscoverProtocol
 
 
@@ -14,6 +15,11 @@ def clear_cache():
 
 
 __all__ = [
-    'AutodiscoverCache', 'AutodiscoverProtocol', 'Autodiscovery', 'discover', 'autodiscover_cache',
-    'close_connections', 'clear_cache'
+    "AutodiscoverCache",
+    "AutodiscoverProtocol",
+    "Autodiscovery",
+    "discover",
+    "autodiscover_cache",
+    "close_connections",
+    "clear_cache",
 ]
