@@ -2,7 +2,7 @@ import logging
 
 from cached_property import threaded_cached_property
 
-from .credentials import BaseCredentials, OAuth2AuthorizationCodeCredentials, OAuth2Credentials, OAuth2LegacyCredentials
+from .credentials import BaseCredentials, BaseOAuth2Credentials
 from .errors import InvalidEnumValue, InvalidTypeError
 from .protocol import FailFast, RetryPolicy
 from .transport import AUTH_TYPE_MAP, CREDENTIALS_REQUIRED, OAUTH2
@@ -11,13 +11,6 @@ from .version import Version
 
 log = logging.getLogger(__name__)
 
-DEFAULT_AUTH_TYPE = {
-    # This type of credentials *must* use the OAuth auth type
-    OAuth2Credentials: OAUTH2,
-    OAuth2LegacyCredentials: OAUTH2,
-    OAuth2AuthorizationCodeCredentials: OAUTH2,
-}
-
 
 class Configuration:
     """Contains information needed to create an authenticated connection to an EWS endpoint.
@@ -59,9 +52,9 @@ class Configuration:
     ):
         if not isinstance(credentials, (BaseCredentials, type(None))):
             raise InvalidTypeError("credentials", credentials, BaseCredentials)
-        if auth_type is None:
+        if auth_type is None and isinstance(credentials, BaseOAuth2Credentials):
             # Set a default auth type for the credentials where this makes sense
-            auth_type = DEFAULT_AUTH_TYPE.get(type(credentials))
+            auth_type = OAUTH2
         if auth_type is not None and auth_type not in AUTH_TYPE_MAP:
             raise InvalidEnumValue("auth_type", auth_type, AUTH_TYPE_MAP)
         if credentials is None and auth_type in CREDENTIALS_REQUIRED:
