@@ -1,5 +1,6 @@
 import abc
 import logging
+from contextlib import suppress
 from fnmatch import fnmatch
 from operator import attrgetter
 
@@ -39,7 +40,7 @@ from ..properties import (
 )
 from ..queryset import DoesNotExist, SearchableMixIn
 from ..util import TNS, is_iterable, require_id
-from ..version import EXCHANGE_2007_SP1, EXCHANGE_2010
+from ..version import EXCHANGE_2007_SP1, EXCHANGE_2010, SupportedVersionClassMixIn
 from .collections import FolderCollection, PullSubscription, PushSubscription, StreamingSubscription, SyncCompleted
 from .queryset import DEEP as DEEP_FOLDERS
 from .queryset import MISSING_FOLDER_ERRORS
@@ -56,7 +57,7 @@ DELETE_FOLDER_ERRORS = (
 )
 
 
-class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
+class BaseFolder(RegisterMixIn, SearchableMixIn, SupportedVersionClassMixIn, metaclass=EWSMeta):
     """Base class for all classes that implement a folder."""
 
     ELEMENT_NAME = "Folder"
@@ -67,9 +68,6 @@ class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
     # https://docs.microsoft.com/en-us/openspecs/exchange_server_protocols/ms-oxosfld/68a85898-84fe-43c4-b166-4711c13cdd61
     CONTAINER_CLASS = None
     supported_item_models = ITEM_CLASSES  # The Item types that this folder can contain. Default is all
-    # Marks the version from which a distinguished folder was introduced. A possibly authoritative source is:
-    # https://github.com/OfficeDev/ews-managed-api/blob/master/Enumerations/WellKnownFolderName.cs
-    supported_from = None
     # Whether this folder type is allowed with the GetFolder service
     get_folder_allowed = True
     DEFAULT_FOLDER_TRAVERSAL_DEPTH = DEEP_FOLDERS
@@ -161,7 +159,7 @@ class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
                 raise ValueError("Already at top")
             yield from self.parent.glob(tail or "*")
         elif head == "**":
-            # Match anything here or in any subfolder at arbitrary depth
+            # Match anything here or in any sub-folder at arbitrary depth
             for c in self.walk():
                 # fnmatch() may be case-sensitive depending on operating system:
                 # force a case-insensitive match since case appears not to
@@ -207,17 +205,10 @@ class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
                 elif i == len(children) and j == 1:
                     # Not the last child, but the first node, which is the name of the child
                     tree += f"└── {node}\n"
-                else:  # Last child, and not name of child
+                else:  # Last child and not name of child
                     tree += f"    {node}\n"
         return tree.strip()
 
-    @classmethod
-    def supports_version(cls, version):
-        # 'version' is a Version instance, for convenience by callers
-        if not cls.supported_from:
-            return True
-        return version.build >= cls.supported_from
-
     @property
     def has_distinguished_name(self):
         return self.name and self.DISTINGUISHED_FOLDER_ID and self.name.lower() == self.DISTINGUISHED_FOLDER_ID.lower()
@@ -329,10 +320,8 @@ class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
     @classmethod
     def get_item_field_by_fieldname(cls, fieldname):
         for item_model in cls.supported_item_models:
-            try:
+            with suppress(InvalidField):
                 return item_model.get_field_by_fieldname(fieldname)
-            except InvalidField:
-                pass
         raise InvalidField(f"{fieldname!r} is not a valid field name on {cls.supported_item_models}")
 
     def get(self, *args, **kwargs):
@@ -422,7 +411,7 @@ class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
             self.root.clear_cache()
 
     def wipe(self, page_size=None, chunk_size=None, _seen=None, _level=0):
-        # Recursively deletes all items in this folder, and all subfolders and their content. Attempts to protect
+        # Recursively deletes all items in this folder, and all sub-folders and their content. Attempts to protect
         # distinguished folders from being deleted. Use with caution!
         from .known_folders import Audits
 
@@ -443,7 +432,7 @@ class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
         has_distinguished_subfolders = any(f.is_distinguished for f in self.children)
         try:
             if has_distinguished_subfolders:
-                self.empty(delete_sub_folders=False)
+                self.empty()
             else:
                 self.empty(delete_sub_folders=True)
         except ErrorRecoverableItemsAccessDenied:
@@ -453,7 +442,7 @@ class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
             try:
                 if has_distinguished_subfolders:
                     raise  # We already tried this
-                self.empty(delete_sub_folders=False)
+                self.empty()
             except DELETE_FOLDER_ERRORS:
                 log.warning("Not allowed to empty %s. Trying to delete items instead", self)
                 kwargs = {}
@@ -468,7 +457,7 @@ class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
         _level += 1
         for f in self.children:
             f.wipe(page_size=page_size, chunk_size=chunk_size, _seen=_seen, _level=_level)
-            # Remove non-distinguished children that are empty and have no subfolders
+            # Remove non-distinguished children that are empty and have no sub-folders
             if f.is_deletable and not f.children:
                 log.warning("Deleting folder %s", f)
                 try:
@@ -485,7 +474,7 @@ class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
 
     @classmethod
     def _kwargs_from_elem(cls, elem, account):
-        # Check for 'DisplayName' element before collecting kwargs because because that clears the elements
+        # Check for 'DisplayName' element before collecting kwargs because that clears the elements
         has_name_elem = elem.find(cls.get_field_by_fieldname("name").response_tag()) is not None
         kwargs = {f.name: f.from_xml(elem=elem, account=account) for f in cls.FIELDS}
         if has_name_elem and not kwargs["name"]:
@@ -705,7 +694,7 @@ class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
         """Get events since the given watermark. Non-blocking.
 
         :param subscription_id: A subscription ID as acquired by .subscribe_to_[pull|push]()
-        :param watermark: Either the watermark from the subscription, or as returned in the last .get_events() call.
+        :param watermark: Either the watermark from the subscription, or as returned by the last .get_events() call.
         :return: A Notification object containing a list of events
 
         This method doesn't need the current folder instance, but it makes sense to keep the method along the other
@@ -754,7 +743,7 @@ class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
 
         Works like as __truediv__ but does not touch the folder cache.
 
-        This is useful if the folder hierarchy contains a huge number of folders and you don't want to fetch them all
+        This is useful if the folder hierarchy contains a huge number of folders, and you don't want to fetch them all
 
         :param other:
         :return:
@@ -914,20 +903,16 @@ class Folder(BaseFolder):
             # The returned XML may contain neither folder class nor name. In that case, we default to the generic
             # Folder class.
             if folder.name:
-                try:
+                with suppress(KeyError):
                     # TODO: fld_class.LOCALIZED_NAMES is most definitely neither complete nor authoritative
                     folder_cls = root.folder_cls_from_folder_name(folder_name=folder.name, locale=root.account.locale)
                     log.debug("Folder class %s matches localized folder name %s", folder_cls, folder.name)
-                except KeyError:
-                    pass
             if folder.folder_class and folder_cls == Folder:
-                try:
+                with suppress(KeyError):
                     folder_cls = cls.folder_cls_from_container_class(container_class=folder.folder_class)
                     log.debug(
                         "Folder class %s matches container class %s (%s)", folder_cls, folder.folder_class, folder.name
                     )
-                except KeyError:
-                    pass
             if folder_cls == Folder:
                 log.debug("Fallback to class Folder (folder_class %s, name %s)", folder.folder_class, folder.name)
         return folder_cls(root=root, **{f.name: getattr(folder, f.name) for f in folder.FIELDS})
