@@ -1,4 +1,5 @@
 import logging
+from contextlib import suppress
 from threading import Lock
 
 from ..errors import ErrorAccessDenied, ErrorFolderNotFound, ErrorInvalidOperation
@@ -39,7 +40,7 @@ class RootOfHierarchy(BaseFolder, metaclass=EWSMeta):
 
     __slots__ = "_account", "_subfolders"
 
-    # A special folder that acts as the top of a folder hierarchy. Finds and caches subfolders at arbitrary depth.
+    # A special folder that acts as the top of a folder hierarchy. Finds and caches sub-folders at arbitrary depth.
     def __init__(self, **kwargs):
         self._account = kwargs.pop("account", None)  # A pointer back to the account holding the folder hierarchy
         super().__init__(**kwargs)
@@ -87,10 +88,8 @@ class RootOfHierarchy(BaseFolder, metaclass=EWSMeta):
     def remove_folder(self, folder):
         if not folder.id:
             raise ValueError("'folder' must have an ID")
-        try:
+        with suppress(KeyError):
             del self._folders_map[folder.id]
-        except KeyError:
-            pass
 
     def clear_cache(self):
         with self._subfolders_lock:
@@ -152,8 +151,8 @@ class RootOfHierarchy(BaseFolder, metaclass=EWSMeta):
             return self._subfolders
 
         with self._subfolders_lock:
-            # Map root, and all subfolders of root, at arbitrary depth by folder ID. First get distinguished folders,
-            # so we are sure to apply the correct Folder class, then fetch all subfolders of this root.
+            # Map root, and all sub-folders of root, at arbitrary depth by folder ID. First get distinguished folders,
+            # so we are sure to apply the correct Folder class, then fetch all sub-folders of this root.
             folders_map = {self.id: self}
             distinguished_folders = [
                 cls(root=self, name=cls.DISTINGUISHED_FOLDER_ID, is_distinguished=True)
@@ -241,10 +240,8 @@ class Root(RootOfHierarchy):
         return self.get_default_folder(MsgFolderRoot)
 
     def get_default_folder(self, folder_cls):
-        try:
+        with suppress(MISSING_FOLDER_ERRORS):
             return super().get_default_folder(folder_cls)
-        except MISSING_FOLDER_ERRORS:
-            pass
 
         # Try to pick a suitable default folder. we do this by:
         #  1. Searching the full folder list for a folder with the distinguished folder name
@@ -262,11 +259,9 @@ class Root(RootOfHierarchy):
 
         # Try direct children of TOIS first, unless we're trying to get the TOIS folder
         if folder_cls != MsgFolderRoot:
-            try:
+            with suppress(MISSING_FOLDER_ERRORS):
                 return self._get_candidate(folder_cls=folder_cls, folder_coll=self.tois.children)
-            except MISSING_FOLDER_ERRORS:
-                # No candidates, or TOIS does not exist, or we don't have access to TOIS
-                pass
+            # No candidates, or TOIS does not exist, or we don't have access to TOIS
 
         # Finally, try direct children of root
         return self._get_candidate(folder_cls=folder_cls, folder_coll=self.children)
@@ -292,7 +287,7 @@ class Root(RootOfHierarchy):
 
 
 class PublicFoldersRoot(RootOfHierarchy):
-    """The root of the public folders hierarchy. Not available on all mailboxes."""
+    """The root of the public folder hierarchy. Not available on all mailboxes."""
 
     DISTINGUISHED_FOLDER_ID = "publicfoldersroot"
     DEFAULT_FOLDER_TRAVERSAL_DEPTH = SHALLOW
@@ -314,7 +309,7 @@ class PublicFoldersRoot(RootOfHierarchy):
             return
 
         children_map = {}
-        try:
+        with suppress(ErrorAccessDenied):
             for f in (
                 SingleFolderQuerySet(account=self.account, folder=folder)
                 .depth(self.DEFAULT_FOLDER_TRAVERSAL_DEPTH)
@@ -326,9 +321,6 @@ class PublicFoldersRoot(RootOfHierarchy):
                 if isinstance(f, Exception):
                     raise f
                 children_map[f.id] = f
-        except ErrorAccessDenied:
-            # No access to this folder
-            pass
 
         # Let's update the cache atomically, to avoid partial reads of the cache.
         with self._subfolders_lock:
