@@ -1,15 +1,47 @@
 import logging
 
-from .base import BaseItem, SAVE_ONLY, SEND_AND_SAVE_COPY, ID_ONLY, SEND_TO_NONE, \
-    AUTO_RESOLVE, SOFT_DELETE, HARD_DELETE, ALL_OCCURRENCIES, MOVE_TO_DELETED_ITEMS
-from ..fields import BooleanField, IntegerField, TextField, CharListField, ChoiceField, URIField, BodyField, \
-    DateTimeField, MessageHeaderField, AttachmentField, Choice, EWSElementField, EffectiveRightsField, CultureField, \
-    CharField, MimeContentField, FieldPath
-from ..properties import ConversationId, ParentFolderId, ReferenceItemId, OccurrenceItemId, RecurringMasterItemId, \
-    ResponseObjects, Fields
-from ..services import GetItem, CreateItem, UpdateItem, DeleteItem, MoveItem, CopyItem, ArchiveItem
+from ..fields import (
+    AttachmentField,
+    BodyField,
+    BooleanField,
+    CharField,
+    CharListField,
+    Choice,
+    ChoiceField,
+    CultureField,
+    DateTimeField,
+    EffectiveRightsField,
+    EWSElementField,
+    FieldPath,
+    IntegerField,
+    MessageHeaderField,
+    MimeContentField,
+    TextField,
+    URIField,
+)
+from ..properties import (
+    ConversationId,
+    Fields,
+    OccurrenceItemId,
+    ParentFolderId,
+    RecurringMasterItemId,
+    ReferenceItemId,
+    ResponseObjects,
+)
 from ..util import is_iterable, require_account, require_id
 from ..version import EXCHANGE_2010, EXCHANGE_2013
+from .base import (
+    ALL_OCCURRENCES,
+    AUTO_RESOLVE,
+    HARD_DELETE,
+    ID_ONLY,
+    MOVE_TO_DELETED_ITEMS,
+    SAVE_ONLY,
+    SEND_AND_SAVE_COPY,
+    SEND_TO_NONE,
+    SOFT_DELETE,
+    BaseItem,
+)
 
 log = logging.getLogger(__name__)
 
@@ -17,62 +49,75 @@ log = logging.getLogger(__name__)
 class Item(BaseItem):
     """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/item"""
 
-    ELEMENT_NAME = 'Item'
+    ELEMENT_NAME = "Item"
 
-    mime_content = MimeContentField(field_uri='item:MimeContent', is_read_only_after_send=True)
-    _id = BaseItem.FIELDS['_id']
-    parent_folder_id = EWSElementField(field_uri='item:ParentFolderId', value_cls=ParentFolderId, is_read_only=True)
-    item_class = CharField(field_uri='item:ItemClass', is_read_only=True)
-    subject = CharField(field_uri='item:Subject')
-    sensitivity = ChoiceField(field_uri='item:Sensitivity', choices={
-        Choice('Normal'), Choice('Personal'), Choice('Private'), Choice('Confidential')
-    }, is_required=True, default='Normal')
-    text_body = TextField(field_uri='item:TextBody', is_read_only=True, supported_from=EXCHANGE_2013)
-    body = BodyField(field_uri='item:Body')  # Accepts and returns Body or HTMLBody instances
-    attachments = AttachmentField(field_uri='item:Attachments')  # ItemAttachment or FileAttachment
-    datetime_received = DateTimeField(field_uri='item:DateTimeReceived', is_read_only=True)
-    size = IntegerField(field_uri='item:Size', is_read_only=True)  # Item size in bytes
-    categories = CharListField(field_uri='item:Categories')
-    importance = ChoiceField(field_uri='item:Importance', choices={
-        Choice('Low'), Choice('Normal'), Choice('High')
-    }, is_required=True, default='Normal')
-    in_reply_to = TextField(field_uri='item:InReplyTo')
-    is_submitted = BooleanField(field_uri='item:IsSubmitted', is_read_only=True)
-    is_draft = BooleanField(field_uri='item:IsDraft', is_read_only=True)
-    is_from_me = BooleanField(field_uri='item:IsFromMe', is_read_only=True)
-    is_resend = BooleanField(field_uri='item:IsResend', is_read_only=True)
-    is_unmodified = BooleanField(field_uri='item:IsUnmodified', is_read_only=True)
-    headers = MessageHeaderField(field_uri='item:InternetMessageHeaders', is_read_only=True)
-    datetime_sent = DateTimeField(field_uri='item:DateTimeSent', is_read_only=True)
-    datetime_created = DateTimeField(field_uri='item:DateTimeCreated', is_read_only=True)
-    response_objects = EWSElementField(field_uri='item:ResponseObjects', value_cls=ResponseObjects,
-                                       is_read_only=True,)
+    mime_content = MimeContentField(field_uri="item:MimeContent", is_read_only_after_send=True)
+    _id = BaseItem.FIELDS["_id"]
+    parent_folder_id = EWSElementField(field_uri="item:ParentFolderId", value_cls=ParentFolderId, is_read_only=True)
+    item_class = CharField(field_uri="item:ItemClass", is_read_only=True)
+    subject = CharField(field_uri="item:Subject")
+    sensitivity = ChoiceField(
+        field_uri="item:Sensitivity",
+        choices={Choice("Normal"), Choice("Personal"), Choice("Private"), Choice("Confidential")},
+        is_required=True,
+        default="Normal",
+    )
+    text_body = TextField(field_uri="item:TextBody", is_read_only=True, supported_from=EXCHANGE_2013)
+    body = BodyField(field_uri="item:Body")  # Accepts and returns Body or HTMLBody instances
+    attachments = AttachmentField(field_uri="item:Attachments")  # ItemAttachment or FileAttachment
+    datetime_received = DateTimeField(field_uri="item:DateTimeReceived", is_read_only=True)
+    size = IntegerField(field_uri="item:Size", is_read_only=True)  # Item size in bytes
+    categories = CharListField(field_uri="item:Categories")
+    importance = ChoiceField(
+        field_uri="item:Importance",
+        choices={Choice("Low"), Choice("Normal"), Choice("High")},
+        is_required=True,
+        default="Normal",
+    )
+    in_reply_to = TextField(field_uri="item:InReplyTo")
+    is_submitted = BooleanField(field_uri="item:IsSubmitted", is_read_only=True)
+    is_draft = BooleanField(field_uri="item:IsDraft", is_read_only=True)
+    is_from_me = BooleanField(field_uri="item:IsFromMe", is_read_only=True)
+    is_resend = BooleanField(field_uri="item:IsResend", is_read_only=True)
+    is_unmodified = BooleanField(field_uri="item:IsUnmodified", is_read_only=True)
+    headers = MessageHeaderField(field_uri="item:InternetMessageHeaders", is_read_only=True)
+    datetime_sent = DateTimeField(field_uri="item:DateTimeSent", is_read_only=True)
+    datetime_created = DateTimeField(field_uri="item:DateTimeCreated", is_read_only=True)
+    response_objects = EWSElementField(
+        field_uri="item:ResponseObjects",
+        value_cls=ResponseObjects,
+        is_read_only=True,
+    )
     # Placeholder for ResponseObjects
-    reminder_due_by = DateTimeField(field_uri='item:ReminderDueBy', is_required_after_save=True, is_searchable=False)
-    reminder_is_set = BooleanField(field_uri='item:ReminderIsSet', is_required=True, default=False)
-    reminder_minutes_before_start = IntegerField(field_uri='item:ReminderMinutesBeforeStart',
-                                                 is_required_after_save=True, min=0, default=0)
-    display_cc = TextField(field_uri='item:DisplayCc', is_read_only=True)
-    display_to = TextField(field_uri='item:DisplayTo', is_read_only=True)
-    has_attachments = BooleanField(field_uri='item:HasAttachments', is_read_only=True)
+    reminder_due_by = DateTimeField(field_uri="item:ReminderDueBy", is_required_after_save=True, is_searchable=False)
+    reminder_is_set = BooleanField(field_uri="item:ReminderIsSet", is_required=True, default=False)
+    reminder_minutes_before_start = IntegerField(
+        field_uri="item:ReminderMinutesBeforeStart", is_required_after_save=True, min=0, default=0
+    )
+    display_cc = TextField(field_uri="item:DisplayCc", is_read_only=True)
+    display_to = TextField(field_uri="item:DisplayTo", is_read_only=True)
+    has_attachments = BooleanField(field_uri="item:HasAttachments", is_read_only=True)
     # ExtendedProperty fields go here
-    culture = CultureField(field_uri='item:Culture', is_required_after_save=True, is_searchable=False)
-    effective_rights = EffectiveRightsField(field_uri='item:EffectiveRights', is_read_only=True)
-    last_modified_name = CharField(field_uri='item:LastModifiedName', is_read_only=True)
-    last_modified_time = DateTimeField(field_uri='item:LastModifiedTime', is_read_only=True)
-    is_associated = BooleanField(field_uri='item:IsAssociated', is_read_only=True, supported_from=EXCHANGE_2010)
-    web_client_read_form_query_string = URIField(field_uri='item:WebClientReadFormQueryString', is_read_only=True,
-                                                 supported_from=EXCHANGE_2010)
-    web_client_edit_form_query_string = URIField(field_uri='item:WebClientEditFormQueryString', is_read_only=True,
-                                                 supported_from=EXCHANGE_2010)
-    conversation_id = EWSElementField(field_uri='item:ConversationId', value_cls=ConversationId,
-                                      is_read_only=True, supported_from=EXCHANGE_2010)
-    unique_body = BodyField(field_uri='item:UniqueBody', is_read_only=True, supported_from=EXCHANGE_2010)
+    culture = CultureField(field_uri="item:Culture", is_required_after_save=True, is_searchable=False)
+    effective_rights = EffectiveRightsField(field_uri="item:EffectiveRights", is_read_only=True)
+    last_modified_name = CharField(field_uri="item:LastModifiedName", is_read_only=True)
+    last_modified_time = DateTimeField(field_uri="item:LastModifiedTime", is_read_only=True)
+    is_associated = BooleanField(field_uri="item:IsAssociated", is_read_only=True, supported_from=EXCHANGE_2010)
+    web_client_read_form_query_string = URIField(
+        field_uri="item:WebClientReadFormQueryString", is_read_only=True, supported_from=EXCHANGE_2010
+    )
+    web_client_edit_form_query_string = URIField(
+        field_uri="item:WebClientEditFormQueryString", is_read_only=True, supported_from=EXCHANGE_2010
+    )
+    conversation_id = EWSElementField(
+        field_uri="item:ConversationId", value_cls=ConversationId, is_read_only=True, supported_from=EXCHANGE_2010
+    )
+    unique_body = BodyField(field_uri="item:UniqueBody", is_read_only=True, supported_from=EXCHANGE_2010)
 
     FIELDS = Fields()
 
     # Used to register extended properties
-    INSERT_AFTER_FIELD = 'has_attachments'
+    INSERT_AFTER_FIELD = "has_attachments"
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
@@ -80,7 +125,7 @@ class Item(BaseItem):
             for a in self.attachments:
                 if a.parent_item:
                     if a.parent_item is not self:
-                        raise ValueError("'parent_item' of attachment %s must point to this item" % a)
+                        raise ValueError(f"'parent_item' of attachment {a} must point to this item")
                 else:
                     a.parent_item = self
                 self.attach(self.attachments)
@@ -89,16 +134,19 @@ class Item(BaseItem):
 
     def save(self, update_fields=None, conflict_resolution=AUTO_RESOLVE, send_meeting_invitations=SEND_TO_NONE):
         from .task import Task
+
         if self.id:
             item_id, changekey = self._update(
                 update_fieldnames=update_fields,
                 message_disposition=SAVE_ONLY,
                 conflict_resolution=conflict_resolution,
-                send_meeting_invitations=send_meeting_invitations
+                send_meeting_invitations=send_meeting_invitations,
             )
-            if self.id != item_id \
-                    and not isinstance(self._id, (OccurrenceItemId, RecurringMasterItemId)) \
-                    and not isinstance(self, Task):
+            if (
+                self.id != item_id
+                and not isinstance(self._id, (OccurrenceItemId, RecurringMasterItemId))
+                and not isinstance(self, Task)
+            ):
                 # When we update an item with an OccurrenceItemId as ID, EWS returns the ID of the occurrence, so
                 # the ID of this item changes.
                 #
@@ -132,6 +180,8 @@ class Item(BaseItem):
     def _create(self, message_disposition, send_meeting_invitations):
         # Return a BulkCreateResult because we want to return the ID of both the main item *and* attachments. In send
         # and send-and-save-copy mode, the server does not return an ID, so we just return True.
+        from ..services import CreateItem
+
         return CreateItem(account=self.account).get(
             items=[self],
             folder=self.folder,
@@ -141,27 +191,28 @@ class Item(BaseItem):
 
     def _update_fieldnames(self):
         from .contact import Contact, DistributionList
+
         # Return the list of fields we are allowed to update
         update_fieldnames = []
         for f in self.supported_fields(version=self.account.version):
-            if f.name == 'attachments':
+            if f.name == "attachments":
                 # Attachments are handled separately after item creation
                 continue
             if f.is_read_only:
                 # These cannot be changed
                 continue
             if (f.is_required or f.is_required_after_save) and (
-                    getattr(self, f.name) is None or (f.is_list and not getattr(self, f.name))
+                getattr(self, f.name) is None or (f.is_list and not getattr(self, f.name))
             ):
                 # These are required and cannot be deleted
                 continue
             if not self.is_draft and f.is_read_only_after_send:
                 # These cannot be changed when the item is no longer a draft
                 continue
-            if f.name == 'message_id' and f.is_read_only_after_send:
+            if f.name == "message_id" and f.is_read_only_after_send:
                 # 'message_id' doesn't support updating, no matter the draft status
                 continue
-            if f.name == 'mime_content' and isinstance(self, (Contact, DistributionList)):
+            if f.name == "mime_content" and isinstance(self, (Contact, DistributionList)):
                 # Contact and DistributionList don't support updating mime_content, no matter the draft status
                 continue
             update_fieldnames.append(f.name)
@@ -169,8 +220,10 @@ class Item(BaseItem):
 
     @require_account
     def _update(self, update_fieldnames, message_disposition, conflict_resolution, send_meeting_invitations):
+        from ..services import UpdateItem
+
         if not self.changekey:
-            raise ValueError('%s must have changekey' % self.__class__.__name__)
+            raise ValueError(f"{self.__class__.__name__} must have changekey")
         if not update_fieldnames:
             # The fields to update was not specified explicitly. Update all fields where update is possible
             update_fieldnames = self._update_fieldnames()
@@ -187,6 +240,8 @@ class Item(BaseItem):
     def refresh(self):
         # Updates the item based on fresh data from EWS
         from ..folders import Folder
+        from ..services import GetItem
+
         additional_fields = {
             FieldPath(field=f) for f in Folder(root=self.account.root).allowed_item_fields(version=self.account.version)
         }
@@ -204,6 +259,8 @@ class Item(BaseItem):
 
     @require_id
     def copy(self, to_folder):
+        from ..services import CopyItem
+
         # If 'to_folder' is a public folder or a folder in a different mailbox then None is returned
         return CopyItem(account=self.account).get(
             items=[self],
@@ -213,6 +270,8 @@ class Item(BaseItem):
 
     @require_id
     def move(self, to_folder):
+        from ..services import MoveItem
+
         res = MoveItem(account=self.account).get(
             items=[self],
             to_folder=to_folder,
@@ -225,31 +284,57 @@ class Item(BaseItem):
         self._id = self.ID_ELEMENT_CLS(*res)
         self.folder = to_folder
 
-    def move_to_trash(self, send_meeting_cancellations=SEND_TO_NONE, affected_task_occurrences=ALL_OCCURRENCIES,
-                      suppress_read_receipts=True):
+    def move_to_trash(
+        self,
+        send_meeting_cancellations=SEND_TO_NONE,
+        affected_task_occurrences=ALL_OCCURRENCES,
+        suppress_read_receipts=True,
+    ):
         # Delete and move to the trash folder.
-        self._delete(delete_type=MOVE_TO_DELETED_ITEMS, send_meeting_cancellations=send_meeting_cancellations,
-                     affected_task_occurrences=affected_task_occurrences, suppress_read_receipts=suppress_read_receipts)
+        self._delete(
+            delete_type=MOVE_TO_DELETED_ITEMS,
+            send_meeting_cancellations=send_meeting_cancellations,
+            affected_task_occurrences=affected_task_occurrences,
+            suppress_read_receipts=suppress_read_receipts,
+        )
         self._id = None
         self.folder = self.account.trash
 
-    def soft_delete(self, send_meeting_cancellations=SEND_TO_NONE, affected_task_occurrences=ALL_OCCURRENCIES,
-                    suppress_read_receipts=True):
+    def soft_delete(
+        self,
+        send_meeting_cancellations=SEND_TO_NONE,
+        affected_task_occurrences=ALL_OCCURRENCES,
+        suppress_read_receipts=True,
+    ):
         # Delete and move to the dumpster, if it is enabled.
-        self._delete(delete_type=SOFT_DELETE, send_meeting_cancellations=send_meeting_cancellations,
-                     affected_task_occurrences=affected_task_occurrences, suppress_read_receipts=suppress_read_receipts)
+        self._delete(
+            delete_type=SOFT_DELETE,
+            send_meeting_cancellations=send_meeting_cancellations,
+            affected_task_occurrences=affected_task_occurrences,
+            suppress_read_receipts=suppress_read_receipts,
+        )
         self._id = None
         self.folder = self.account.recoverable_items_deletions
 
-    def delete(self, send_meeting_cancellations=SEND_TO_NONE, affected_task_occurrences=ALL_OCCURRENCIES,
-               suppress_read_receipts=True):
+    def delete(
+        self,
+        send_meeting_cancellations=SEND_TO_NONE,
+        affected_task_occurrences=ALL_OCCURRENCES,
+        suppress_read_receipts=True,
+    ):
         # Remove the item permanently. No copies are stored anywhere.
-        self._delete(delete_type=HARD_DELETE, send_meeting_cancellations=send_meeting_cancellations,
-                     affected_task_occurrences=affected_task_occurrences, suppress_read_receipts=suppress_read_receipts)
+        self._delete(
+            delete_type=HARD_DELETE,
+            send_meeting_cancellations=send_meeting_cancellations,
+            affected_task_occurrences=affected_task_occurrences,
+            suppress_read_receipts=suppress_read_receipts,
+        )
         self._id, self.folder = None, None
 
     @require_id
     def _delete(self, delete_type, send_meeting_cancellations, affected_task_occurrences, suppress_read_receipts):
+        from ..services import DeleteItem
+
         DeleteItem(account=self.account).get(
             items=[self],
             delete_type=delete_type,
@@ -260,6 +345,8 @@ class Item(BaseItem):
 
     @require_id
     def archive(self, to_folder):
+        from ..services import ArchiveItem
+
         return ArchiveItem(account=self.account).get(items=[self], to_folder=to_folder, expect_result=True)
 
     def attach(self, attachments):
@@ -298,7 +385,7 @@ class Item(BaseItem):
             attachments = list(attachments)
         for a in attachments:
             if a.parent_item is not self:
-                raise ValueError('Attachment does not belong to this item')
+                raise ValueError("Attachment does not belong to this item")
             if self.id:
                 # Item is already created. Detach  the attachment server-side now
                 a.detach()
@@ -308,6 +395,7 @@ class Item(BaseItem):
     @require_id
     def create_forward(self, subject, body, to_recipients, cc_recipients=None, bcc_recipients=None):
         from .message import ForwardItem
+
         return ForwardItem(
             account=self.account,
             reference_item_id=ReferenceItemId(id=self.id, changekey=self.changekey),
