@@ -14,10 +14,10 @@ from threading import Lock
 
 import requests.adapters
 import requests.sessions
-from oauthlib.oauth2 import BackendApplicationClient, LegacyApplicationClient, WebApplicationClient
+from oauthlib.oauth2 import BackendApplicationClient
 from requests_oauthlib import OAuth2Session
 
-from .credentials import OAuth2AuthorizationCodeCredentials, OAuth2Credentials, OAuth2LegacyCredentials
+from .credentials import BaseOAuth2Credentials
 from .errors import (
     CASError,
     ErrorInvalidSchemaVersionForMailboxVersion,
@@ -29,7 +29,17 @@ from .errors import (
     TransportError,
     UnauthorizedError,
 )
-from .properties import DLMailbox, FreeBusyViewOptions, MailboxData, RoomList, TimeWindow, TimeZone
+from .properties import (
+    ENTRY_ID,
+    EWS_ID,
+    AlternateId,
+    DLMailbox,
+    FreeBusyViewOptions,
+    MailboxData,
+    RoomList,
+    TimeWindow,
+    TimeZone,
+)
 from .services import (
     ConvertId,
     ExpandDL,
@@ -41,7 +51,7 @@ from .services import (
     ResolveNames,
 )
 from .transport import CREDENTIALS_REQUIRED, DEFAULT_HEADERS, NTLM, OAUTH2, get_auth_instance, get_service_authtype
-from .version import API_VERSIONS, Version
+from .version import Version
 
 log = logging.getLogger(__name__)
 
@@ -77,7 +87,6 @@ class BaseProtocol:
 
     def __init__(self, config):
         self.config = config
-        self._api_version_hint = None
 
         self._session_pool_size = 0
         self._session_pool_maxsize = config.max_connections or self.SESSION_POOLSIZE
@@ -92,6 +101,10 @@ class BaseProtocol:
     def service_endpoint(self):
         return self.config.service_endpoint
 
+    @abc.abstractmethod
+    def get_auth_type(self):
+        """Autodetect authentication type"""
+
     @property
     def auth_type(self):
         # Autodetect authentication type if necessary
@@ -119,15 +132,6 @@ class BaseProtocol:
     def server(self):
         return self.config.server
 
-    def get_auth_type(self):
-        # Autodetect authentication type. We also set version hint here.
-        name = str(self.credentials) if self.credentials and str(self.credentials) else "DUMMY"
-        auth_type, api_version_hint = get_service_authtype(
-            service_endpoint=self.service_endpoint, retry_policy=self.retry_policy, api_versions=API_VERSIONS, name=name
-        )
-        self._api_version_hint = api_version_hint
-        return auth_type
-
     def __getstate__(self):
         # The session pool and lock cannot be pickled
         state = self.__dict__.copy()
@@ -245,11 +249,12 @@ class BaseProtocol:
         self._session_pool.put(session, block=False)
 
     def close_session(self, session):
-        if isinstance(self.credentials, OAuth2Credentials) and not isinstance(
-            self.credentials, OAuth2AuthorizationCodeCredentials
+        if isinstance(self.credentials, BaseOAuth2Credentials) and isinstance(
+            self.credentials.client, BackendApplicationClient
         ):
-            # Reset token if client is of type BackendApplicationClient
-            self.credentials.access_token = None
+            # Reset access token
+            with self.credentials.lock:
+                self.credentials.access_token = None
         session.close()
         del session
 
@@ -286,24 +291,22 @@ class BaseProtocol:
             session = self.raw_session(self.service_endpoint)
             session.auth = get_auth_instance(auth_type=self.auth_type)
         else:
-            with self.credentials.lock:
-                if isinstance(self.credentials, OAuth2Credentials):
+            if isinstance(self.credentials, BaseOAuth2Credentials):
+                with self.credentials.lock:
                     session = self.create_oauth2_session()
-                    # Keep track of the credentials used to create this session. If
-                    # and when we need to renew credentials (for example, refreshing
-                    # an OAuth access token), this lets us easily determine whether
-                    # the credentials have already been refreshed in another thread
-                    # by the time this session tries.
+                    # Keep track of the credentials used to create this session. If and when we need to renew
+                    # credentials (for example, refreshing an OAuth access token), this lets us easily determine whether
+                    # the credentials have already been refreshed in another thread by the time this session tries.
                     session.credentials_sig = self.credentials.sig()
+            else:
+                if self.auth_type == NTLM and self.credentials.type == self.credentials.EMAIL:
+                    username = "\\" + self.credentials.username
                 else:
-                    if self.auth_type == NTLM and self.credentials.type == self.credentials.EMAIL:
-                        username = "\\" + self.credentials.username
-                    else:
-                        username = self.credentials.username
-                    session = self.raw_session(self.service_endpoint)
-                    session.auth = get_auth_instance(
-                        auth_type=self.auth_type, username=username, password=self.credentials.password
-                    )
+                    username = self.credentials.username
+                session = self.raw_session(self.service_endpoint)
+                session.auth = get_auth_instance(
+                    auth_type=self.auth_type, username=username, password=self.credentials.password
+                )
 
         # Add some extra info
         session.session_id = random.randint(10000, 99999)  # Used for debugging messages in services
@@ -312,40 +315,10 @@ class BaseProtocol:
         return session
 
     def create_oauth2_session(self):
-        session_params = {"token": self.credentials.access_token}  # Token may be None
-        token_params = {"include_client_id": True}
-
-        if isinstance(self.credentials, OAuth2AuthorizationCodeCredentials):
-            token_params["code"] = self.credentials.authorization_code  # Auth code may be None
-            self.credentials.authorization_code = None  # We can only use the code once
-
-            if self.credentials.client_id and self.credentials.client_secret:
-                # If we're given a client ID and secret, we have enough to refresh access tokens ourselves. In other
-                # cases the session will raise TokenExpiredError, and we'll need to ask the calling application to
-                # refresh the token (that covers cases where the caller doesn't have access to the client secret but
-                # is working with a service that can provide it refreshed tokens on a limited basis).
-                session_params.update(
-                    {
-                        "auto_refresh_kwargs": {
-                            "client_id": self.credentials.client_id,
-                            "client_secret": self.credentials.client_secret,
-                        },
-                        "auto_refresh_url": self.credentials.token_url,
-                        "token_updater": self.credentials.on_token_auto_refreshed,
-                    }
-                )
-            client = WebApplicationClient(client_id=self.credentials.client_id)
-        elif isinstance(self.credentials, OAuth2LegacyCredentials):
-            client = LegacyApplicationClient(client_id=self.credentials.client_id)
-            token_params["username"] = self.credentials.username
-            token_params["password"] = self.credentials.password
-        else:
-            client = BackendApplicationClient(client_id=self.credentials.client_id)
-
         session = self.raw_session(
             self.service_endpoint,
-            oauth2_client=client,
-            oauth2_session_params=session_params,
+            oauth2_client=self.credentials.client,
+            oauth2_session_params=self.credentials.session_params(),
             oauth2_token_endpoint=self.credentials.token_url,
         )
         if not session.token:
@@ -356,12 +329,12 @@ class BaseProtocol:
                 client_secret=self.credentials.client_secret,
                 scope=self.credentials.scope,
                 timeout=self.TIMEOUT,
-                **token_params,
+                **self.credentials.token_params(),
             )
             # Allow the credentials object to update its copy of the new token, and give the application an opportunity
             # to cache it.
             self.credentials.on_token_auto_refreshed(token)
-        session.auth = get_auth_instance(auth_type=OAUTH2, client=client)
+        session.auth = get_auth_instance(auth_type=OAUTH2, client=self.credentials.client)
 
         return session
 
@@ -415,7 +388,7 @@ class CachingProtocol(type):
             return protocol
 
         # Acquire lock to guard against multiple threads competing to cache information. Having a per-server lock is
-        # probably overkill although it would reduce lock contention.
+        # probably overkill, although it would reduce lock contention.
         log.debug("Waiting for _protocol_cache_lock")
         with cls._protocol_cache_lock:
             try:
@@ -474,15 +447,21 @@ class Protocol(BaseProtocol, metaclass=CachingProtocol):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self._version_lock = Lock()
+        self.api_version_hint = None
+
+    def get_auth_type(self):
+        # Autodetect authentication type. We also set 'self.api_version_hint' here.
+        return get_service_authtype(protocol=self)
 
     @property
     def version(self):
         # Make sure only one thread does the guessing.
         if not self.config.version or not self.config.version.build:
+            log.debug("Waiting for _version_lock")
             with self._version_lock:
                 if not self.config.version or not self.config.version.build:
                     # Version.guess() needs auth objects and a working session pool
-                    self.config.version = Version.guess(self, api_version_hint=self._api_version_hint)
+                    self.config.version = Version.guess(self, api_version_hint=self.api_version_hint)
         return self.config.version
 
     def get_timezones(self, timezones=None, return_full_timezone_data=False):
@@ -560,7 +539,7 @@ class Protocol(BaseProtocol, metaclass=CachingProtocol):
         )
 
     def expand_dl(self, distribution_list):
-        """Expand distribution list into it's members.
+        """Expand distribution list into its members.
 
         :param distribution_list: SMTP address of the distribution list to expand, or a DLMailbox representing the list
 
@@ -598,6 +577,17 @@ class Protocol(BaseProtocol, metaclass=CachingProtocol):
         """
         return ConvertId(protocol=self).call(items=ids, destination_format=destination_format)
 
+    def dummy_xml(self):
+        # Generate a minimal, valid EWS request
+        svc = ConvertId(protocol=self)
+        return svc.wrap(
+            content=svc.get_payload(
+                items=[AlternateId(id="DUMMY", format=EWS_ID, mailbox="DUMMY")],
+                destination_format=ENTRY_ID,
+            ),
+            api_version=self.api_version_hint,
+        )
+
     def __getstate__(self):
         # The lock cannot be pickled
         state = super().__getstate__()
@@ -629,7 +619,7 @@ class NoVerifyHTTPAdapter(requests.adapters.HTTPAdapter):
 
     def cert_verify(self, conn, url, verify, cert):
         # pylint: disable=unused-argument
-        # We're overriding a method so we have to keep the signature
+        # We're overriding a method, so we have to keep the signature
         super().cert_verify(conn=conn, url=url, verify=False, cert=cert)
 
 
@@ -713,7 +703,7 @@ class FailFast(RetryPolicy):
         raise ValueError("Cannot back off with fail-fast policy")
 
     def may_retry_on_error(self, response, wait):
-        log.debug("No retry: no fail-fast policy")
+        log.debug("No retry with fail-fast policy")
         return False
 
 
