@@ -1,5 +1,6 @@
 import abc
 import logging
+from contextlib import suppress
 from copy import deepcopy
 from itertools import islice
 
@@ -44,7 +45,7 @@ class SearchableMixIn:
 
 
 class QuerySet(SearchableMixIn):
-    """A Django QuerySet-like class for querying items. Defers queries until the QuerySet is consumed. Supports
+    """A Django QuerySet-like class for querying items. Defers query until the QuerySet is consumed. Supports
     chaining to build up complex queries.
 
     Django QuerySet documentation: https://docs.djangoproject.com/en/dev/ref/models/querysets/
@@ -110,10 +111,8 @@ class QuerySet(SearchableMixIn):
         if self.request_type == self.PERSONA:
             return FieldPath(field=Persona.get_field_by_fieldname(field_path))
         for folder in self.folder_collection:
-            try:
+            with suppress(InvalidField):
                 return FieldPath.from_string(field_path=field_path, folder=folder)
-            except InvalidField:
-                pass
         raise InvalidField(f"Unknown field path {field_path!r} on folders {self.folder_collection.folders}")
 
     def _get_field_order(self, field_path):
@@ -125,10 +124,8 @@ class QuerySet(SearchableMixIn):
                 reverse=field_path.startswith("-"),
             )
         for folder in self.folder_collection:
-            try:
+            with suppress(InvalidField):
                 return FieldOrder.from_string(field_path=field_path, folder=folder)
-            except InvalidField:
-                pass
         raise InvalidField(f"Unknown field path {field_path!r} on folders {self.folder_collection.folders}")
 
     @property
@@ -305,7 +302,7 @@ class QuerySet(SearchableMixIn):
         raise IndexError()
 
     def _getitem_slice(self, s):
-        from .services import PAGE_SIZE
+        from .services import FindItem
 
         if ((s.start or 0) < 0) or ((s.stop or 0) < 0) or ((s.step or 0) < 0):
             # islice() does not support negative start, stop and step. Make sure cache is full by iterating the full
@@ -320,7 +317,7 @@ class QuerySet(SearchableMixIn):
             new_qs.offset = s.start
         elif s.stop is not None:
             new_qs.max_items = s.stop
-        if new_qs.page_size is None and new_qs.max_items is not None and new_qs.max_items < PAGE_SIZE:
+        if new_qs.page_size is None and new_qs.max_items is not None and new_qs.max_items < FindItem.PAGE_SIZE:
             new_qs.page_size = new_qs.max_items
         return islice(new_qs.__iter__(), None, None, s.step)
 
