@@ -5,7 +5,7 @@ from itertools import chain
 
 from .. import errors
 from ..attachments import AttachmentId
-from ..credentials import IMPERSONATION, OAuth2Credentials
+from ..credentials import IMPERSONATION, BaseOAuth2Credentials
 from ..errors import (
     ErrorBatchProcessingStopped,
     ErrorCannotDeleteObject,
@@ -13,6 +13,7 @@ from ..errors import (
     ErrorCorruptData,
     ErrorExceededConnectionCount,
     ErrorIncorrectSchemaVersion,
+    ErrorInternalServerTransientError,
     ErrorInvalidChangeKey,
     ErrorInvalidIdMalformed,
     ErrorInvalidRequest,
@@ -47,7 +48,7 @@ from ..properties import (
     IndexedFieldURI,
     ItemId,
 )
-from ..transport import wrap
+from ..transport import DEFAULT_ENCODING
 from ..util import (
     ENS,
     MNS,
@@ -59,25 +60,24 @@ from ..util import (
     chunkify,
     create_element,
     get_xml_attr,
+    ns_translation,
     post_ratelimited,
     set_xml_value,
     to_xml,
     xml_to_str,
 )
-from ..version import API_VERSIONS, Version
+from ..version import SupportedVersionClassMixIn, Version
 
 log = logging.getLogger(__name__)
 
 
-class EWSService(metaclass=abc.ABCMeta):
+class EWSService(SupportedVersionClassMixIn, metaclass=abc.ABCMeta):
     """Base class for all EWS services."""
 
-    PAGE_SIZE = 100  # A default page size for all paging services. This is the number of items we request per page
     CHUNK_SIZE = 100  # A default chunk size for all services. This is the number of items we send in a single request
 
     SERVICE_NAME = None  # The name of the SOAP service
     element_container_name = None  # The name of the XML element wrapping the collection of returned items
-    paging_container_name = None  # The name of the element that contains paging information and the paged results
     returns_elements = True  # If False, the service does not return response elements, just the ResponseCode status
     # Return exception instance instead of raising exceptions for the following errors when contained in an element
     ERRORS_TO_CATCH_IN_RESPONSE = (
@@ -95,16 +95,14 @@ class EWSService(metaclass=abc.ABCMeta):
         ErrorItemCorrupt,
         ErrorMailRecipientNotFound,
     )
-    # Similarly, define the warnings we want to return unraised
+    # Similarly, define the warnings we want to return un-raised
     WARNINGS_TO_CATCH_IN_RESPONSE = ErrorBatchProcessingStopped
     # Define the warnings we want to ignore, to let response processing proceed
     WARNINGS_TO_IGNORE_IN_RESPONSE = ()
     # The exception type to raise when all attempted API versions failed
     NO_VALID_SERVER_VERSIONS = ErrorInvalidServerVersion
-    # Marks the version from which the service was introduced
-    supported_from = None
-    # Marks services that support paging of requested items
-    supports_paging = False
+
+    NS_MAP = {k: v for k, v in ns_translation.items() if k in ("s", "m", "t")}
 
     def __init__(self, protocol, chunk_size=None, timeout=None):
         self.chunk_size = chunk_size or self.CHUNK_SIZE
@@ -114,8 +112,8 @@ class EWSService(metaclass=abc.ABCMeta):
             raise ValueError(f"'chunk_size' {self.chunk_size} must be a positive number")
         if self.supported_from and protocol.version.build < self.supported_from:
             raise NotImplementedError(
-                f"{self.SERVICE_NAME!r} is only supported on {self.supported_from.fullname()!r} and later. "
-                f"Your current version is {protocol.version.build.fullname()!r}."
+                f"Service {self.SERVICE_NAME!r} only supports server versions from {self.supported_from or '*'} to "
+                f"{self.deprecated_from or '*'} (server has {protocol.version})"
             )
         self.protocol = protocol
         # Allow a service to override the default protocol timeout. Useful for streaming services
@@ -183,6 +181,43 @@ class EWSService(metaclass=abc.ABCMeta):
         _, body = self._get_soap_parts(response=resp)
         return self._elems_to_objs(self._get_elements_in_response(response=self._get_soap_messages(body=body)))
 
+    def wrap(self, content, api_version=None):
+        """Generate the necessary boilerplate XML for a raw SOAP request. The XML is specific to the server version.
+        ExchangeImpersonation allows to act as the user we want to impersonate.
+
+        RequestServerVersion element on MSDN:
+        https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/requestserverversion
+
+        ExchangeImpersonation element on MSDN:
+        https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/exchangeimpersonation
+
+        TimeZoneContent element on MSDN:
+        https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/timezonecontext
+
+        :param content:
+        :param api_version:
+        """
+        envelope = create_element("s:Envelope", nsmap=self.NS_MAP)
+        header = create_element("s:Header")
+        if api_version:
+            request_server_version = create_element("t:RequestServerVersion", attrs=dict(Version=api_version))
+            header.append(request_server_version)
+        identity = self._account_to_impersonate
+        if identity:
+            add_xml_child(header, "t:ExchangeImpersonation", identity)
+        timezone = self._timezone
+        if timezone:
+            timezone_context = create_element("t:TimeZoneContext")
+            timezone_definition = create_element("t:TimeZoneDefinition", attrs=dict(Id=timezone.ms_id))
+            timezone_context.append(timezone_definition)
+            header.append(timezone_context)
+        if len(header):
+            envelope.append(header)
+        body = create_element("s:Body")
+        body.append(content)
+        envelope.append(body)
+        return xml_to_str(envelope, encoding=DEFAULT_ENCODING, xml_declaration=True)
+
     def _elems_to_objs(self, elems):
         """Takes a generator of XML elements and exceptions. Returns the equivalent Python objects (or exceptions)."""
         for elem in elems:
@@ -206,12 +241,18 @@ class EWSService(metaclass=abc.ABCMeta):
     def _version_hint(self, value):
         self.protocol.config.version = value
 
-    def _extra_headers(self, session):
-        return {}
+    def _extra_headers(self):
+        headers = {}
+        identity = self._account_to_impersonate
+        if identity and identity.primary_smtp_address:
+            # See
+            # https://blogs.msdn.microsoft.com/webdav_101/2015/05/11/best-practices-ews-authentication-and-access-issues/
+            headers["X-AnchorMailbox"] = identity.primary_smtp_address
+        return headers
 
     @property
     def _account_to_impersonate(self):
-        if isinstance(self.protocol.credentials, OAuth2Credentials):
+        if self.protocol and isinstance(self.protocol.credentials, BaseOAuth2Credentials):
             return self.protocol.credentials.identity
         return None
 
@@ -226,8 +267,6 @@ class EWSService(metaclass=abc.ABCMeta):
         :return: the response, as XML objects
         """
         response = self._get_response_xml(payload=payload)
-        if self.supports_paging:
-            return (self._get_page(message) for message in response)
         return self._get_elements_in_response(response=response)
 
     def _chunked_get_elements(self, payload_func, items, **kwargs):
@@ -240,7 +279,7 @@ class EWSService(metaclass=abc.ABCMeta):
         :return: Same as ._get_elements()
         """
         # If the input for a service is a QuerySet, it can be difficult to remove exceptions before now
-        filtered_items = filter(lambda i: not isinstance(i, Exception), items)
+        filtered_items = filter(lambda item: not isinstance(item, Exception), items)
         for i, chunk in enumerate(chunkify(filtered_items, self.chunk_size), start=1):
             log.debug("Processing chunk %s containing %s items", i, len(chunk))
             yield from self._get_elements(payload=payload_func(chunk, **kwargs))
@@ -268,7 +307,7 @@ class EWSService(metaclass=abc.ABCMeta):
             except ErrorServerBusy as e:
                 self._handle_backoff(e)
                 continue
-            except (ErrorTooManyObjectsOpened, ErrorTimeoutExpired) as e:
+            except (ErrorTooManyObjectsOpened, ErrorTimeoutExpired, ErrorInternalServerTransientError) as e:
                 # ErrorTooManyObjectsOpened means there are too many connections to the Exchange database. This is very
                 # often a symptom of sending too many requests.
                 #
@@ -299,12 +338,10 @@ class EWSService(metaclass=abc.ABCMeta):
             protocol=self.protocol,
             session=session,
             url=self.protocol.service_endpoint,
-            headers=self._extra_headers(session),
-            data=wrap(
+            headers=self._extra_headers(),
+            data=self.wrap(
                 content=payload,
                 api_version=api_version,
-                account_to_impersonate=self._account_to_impersonate,
-                timezone=self._timezone,
             ),
             stream=self.streaming,
             timeout=self.timeout or self.protocol.TIMEOUT,
@@ -318,10 +355,16 @@ class EWSService(metaclass=abc.ABCMeta):
             self.protocol.release_session(session)
         return r
 
-    @property
+    @classmethod
+    def supported_api_versions(cls):
+        """Return API versions supported by the service, sorted from newest to oldest"""
+        return sorted({v.api_version for v in Version.all_versions() if cls.supports_version(v)}, reverse=True)
+
     def _api_versions_to_try(self):
         # Put the hint first in the list, and then all other versions except the hint, from newest to oldest
-        return (self._version_hint.api_version,) + tuple(v for v in API_VERSIONS if v != self._version_hint.api_version)
+        return (self._version_hint.api_version,) + tuple(
+            v for v in self.supported_api_versions() if v != self._version_hint.api_version
+        )
 
     def _get_response_xml(self, payload, **parse_opts):
         """Send the payload to the server and return relevant elements from the result. Several things happen here:
@@ -338,7 +381,7 @@ class EWSService(metaclass=abc.ABCMeta):
         # guessing tango, but then the server may decide that any arbitrary legacy backend server may actually process
         # the request for an account. Prepare to handle version-related errors and set the server version per-account.
         log.debug("Calling service %s", self.SERVICE_NAME)
-        for api_version in self._api_versions_to_try:
+        for api_version in self._api_versions_to_try():
             log.debug("Trying API version %s", api_version)
             r = self._get_response(payload=payload, api_version=api_version)
             if self.streaming:
@@ -377,7 +420,7 @@ class EWSService(metaclass=abc.ABCMeta):
                     # In streaming mode, we may not have accessed the raw stream yet. Caller must handle this.
                     r.close()  # Release memory
 
-        raise self.NO_VALID_SERVER_VERSIONS(f"Tried versions {self._api_versions_to_try} but all were invalid")
+        raise self.NO_VALID_SERVER_VERSIONS(f"Tried versions {self._api_versions_to_try()} but all were invalid")
 
     def _handle_backoff(self, e):
         """Take a request from the server to back off and checks the retry policy for what to do. Re-raise the
@@ -408,7 +451,7 @@ class EWSService(metaclass=abc.ABCMeta):
             # Nothing to do
             return
         log.debug("Found new version (%s -> %s)", self._version_hint, head_version)
-        # The api_version that worked was different than our hint, or we never got a build version. Store the working
+        # The api_version that worked was different from our hint, or we never got a build version. Store the working
         # version.
         self._version_hint = head_version
 
@@ -429,7 +472,7 @@ class EWSService(metaclass=abc.ABCMeta):
 
     @classmethod
     def _get_soap_parts(cls, response, **parse_opts):
-        """Split the SOAP response into its headers an body elements."""
+        """Split the SOAP response into its headers and body elements."""
         try:
             root = to_xml(response.iter_content())
         except ParseError as e:
@@ -548,7 +591,7 @@ class EWSService(metaclass=abc.ABCMeta):
                 if container is None:
                     raise MalformedResponseError(f"No {name} elements in ResponseMessage ({xml_to_str(message)})")
                 return container
-        # rspclass == 'Error', or 'Success' and not 'NoError'
+        # response_class == 'Error', or 'Success' and not 'NoError'
         try:
             raise self._get_exception(code=response_code, text=msg_text, msg_xml=msg_xml)
         except self.ERRORS_TO_CATCH_IN_RESPONSE as e:
@@ -638,7 +681,7 @@ class EWSService(metaclass=abc.ABCMeta):
     @classmethod
     def _get_elements_in_container(cls, container):
         """Return a list of response elements from an XML response element container. With e.g.
-        'CreateItem', 'Items' is the container element and we return the 'Message' child elements:
+        'CreateItem', 'Items' is the container element, and we return the 'Message' child elements:
 
           <m:Items>
             <t:Message>
@@ -684,8 +727,8 @@ class EWSAccountService(EWSService, metaclass=abc.ABCMeta):
                     self.account.affinity_cookie = cookie.value
                     break
 
-    def _extra_headers(self, session):
-        headers = super()._extra_headers(session=session)
+    def _extra_headers(self):
+        headers = super()._extra_headers()
         # See
         # https://blogs.msdn.microsoft.com/webdav_101/2015/05/11/best-practices-ews-authentication-and-access-issues/
         headers["X-AnchorMailbox"] = self.account.primary_smtp_address
@@ -710,6 +753,10 @@ class EWSAccountService(EWSService, metaclass=abc.ABCMeta):
 
 
 class EWSPagingService(EWSAccountService):
+    PAGE_SIZE = 100  # A default page size for all paging services. This is the number of items we request per page
+
+    paging_container_name = None  # The name of the element that contains paging information and the paged results
+
     def __init__(self, *args, **kwargs):
         self.page_size = kwargs.pop("page_size", None) or self.PAGE_SIZE
         if not isinstance(self.page_size, int):
@@ -718,6 +765,15 @@ class EWSPagingService(EWSAccountService):
             raise ValueError(f"'page_size' {self.page_size} must be a positive number")
         super().__init__(*args, **kwargs)
 
+    def _response_generator(self, payload):
+        """Send the payload to the server, and return the response.
+
+        :param payload: payload as an XML object
+        :return: the response, as XML objects
+        """
+        response = self._get_response_xml(payload=payload)
+        return (self._get_page(message) for message in response)
+
     def _paged_call(self, payload_func, max_items, folders, **kwargs):
         """Call a service that supports paging requests. Return a generator over all response items. Keeps track of
         all paging-related counters.
@@ -843,7 +899,7 @@ class EWSPagingService(EWSAccountService):
             # Paging is done for all messages
             return None
         # We cannot guarantee that all messages that have a next_offset also have the *same* next_offset. This is
-        # because the collections that we are iterating may change while iterating. We'll do our best but we cannot
+        # because the collections that we are iterating may change while iterating. We'll do our best, but we cannot
         # guarantee 100% consistency when large collections are simultaneously being changed on the server.
         #
         # It's not possible to supply a per-folder offset when iterating multiple folders, so we'll just have to
