@@ -1,22 +1,13 @@
 import logging
 import time
+from contextlib import suppress
 
 import requests.auth
 import requests_ntlm
 import requests_oauthlib
 
 from .errors import TransportError, UnauthorizedError
-from .util import (
-    CONNECTION_ERRORS,
-    RETRY_WAIT,
-    DummyResponse,
-    _back_off_if_needed,
-    _retry_after,
-    add_xml_child,
-    create_element,
-    ns_translation,
-    xml_to_str,
-)
+from .util import CONNECTION_ERRORS, RETRY_WAIT, TLS_ERRORS, DummyResponse, _back_off_if_needed, _retry_after
 
 log = logging.getLogger(__name__)
 
@@ -41,78 +32,21 @@ AUTH_TYPE_MAP = {
     CBA: None,
     NOAUTH: None,
 }
-try:
+with suppress(ImportError):
+    # Kerberos auth is optional
     import requests_gssapi
 
     AUTH_TYPE_MAP[GSSAPI] = requests_gssapi.HTTPSPNEGOAuth
-except ImportError:
-    # Kerberos auth is optional
-    pass
-try:
+with suppress(ImportError):
+    # SSPI auth is optional
     import requests_negotiate_sspi
 
     AUTH_TYPE_MAP[SSPI] = requests_negotiate_sspi.HttpNegotiateAuth
-except ImportError:
-    # SSPI auth is optional
-    pass
 
 DEFAULT_ENCODING = "utf-8"
 DEFAULT_HEADERS = {"Content-Type": f"text/xml; charset={DEFAULT_ENCODING}", "Accept-Encoding": "gzip, deflate"}
 
 
-def wrap(content, api_version=None, account_to_impersonate=None, timezone=None):
-    """Generate the necessary boilerplate XML for a raw SOAP request. The XML is specific to the server version.
-    ExchangeImpersonation allows to act as the user we want to impersonate.
-
-    RequestServerVersion element on MSDN:
-    https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/requestserverversion
-
-    ExchangeImpersonation element on MSDN:
-    https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/exchangeimpersonation
-
-    TimeZoneContent element on MSDN:
-    https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/timezonecontext
-
-    :param content:
-    :param api_version:
-    :param account_to_impersonate:  (Default value = None)
-    :param timezone:  (Default value = None)
-    """
-    envelope = create_element("s:Envelope", nsmap=ns_translation)
-    header = create_element("s:Header")
-    if api_version:
-        request_server_version = create_element("t:RequestServerVersion", attrs=dict(Version=api_version))
-        header.append(request_server_version)
-    if account_to_impersonate:
-        exchange_impersonation = create_element("t:ExchangeImpersonation")
-        connecting_sid = create_element("t:ConnectingSID")
-        # We have multiple options for uniquely identifying the user. Here's a prioritized list in accordance with
-        # https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/connectingsid
-        for attr, tag in (
-            ("sid", "SID"),
-            ("upn", "PrincipalName"),
-            ("smtp_address", "SmtpAddress"),
-            ("primary_smtp_address", "PrimarySmtpAddress"),
-        ):
-            val = getattr(account_to_impersonate, attr)
-            if val:
-                add_xml_child(connecting_sid, f"t:{tag}", val)
-                break
-        exchange_impersonation.append(connecting_sid)
-        header.append(exchange_impersonation)
-    if timezone:
-        timezone_context = create_element("t:TimeZoneContext")
-        timezone_definition = create_element("t:TimeZoneDefinition", attrs=dict(Id=timezone.ms_id))
-        timezone_context.append(timezone_definition)
-        header.append(timezone_context)
-    if len(header):
-        envelope.append(header)
-    body = create_element("s:Body")
-    body.append(content)
-    envelope.append(body)
-    return xml_to_str(envelope, encoding=DEFAULT_ENCODING, xml_declaration=True)
-
-
 def get_auth_instance(auth_type, **kwargs):
     """Return an *Auth instance suitable for the requests package.
 
@@ -131,31 +65,34 @@ def get_auth_instance(auth_type, **kwargs):
     return model(**kwargs)
 
 
-def get_service_authtype(service_endpoint, retry_policy, api_versions, name):
-    # Get auth type by tasting headers from the server. Only do POST requests. HEAD is too error prone, and some servers
+def get_service_authtype(protocol):
+    # Get auth type by tasting headers from the server. Only do POST requests. HEAD is too error-prone, and some servers
     # are set up to redirect to OWA on all requests except POST to /EWS/Exchange.asmx
     #
     # We don't know the API version yet, but we need it to create a valid request because some Exchange servers only
     # respond when given a valid request. Try all known versions. Gross.
-    from .protocol import BaseProtocol
+    from .services import ConvertId
 
+    service_endpoint = protocol.service_endpoint
+    retry_policy = protocol.retry_policy
     retry = 0
     t_start = time.monotonic()
-    headers = DEFAULT_HEADERS.copy()
-    for api_version in api_versions:
-        data = dummy_xml(api_version=api_version, name=name)
+    for api_version in ConvertId.supported_api_versions():
+        protocol.api_version_hint = api_version
+        data = protocol.dummy_xml()
+        headers = {}
         log.debug("Requesting %s from %s", data, service_endpoint)
         while True:
             _back_off_if_needed(retry_policy.back_off_until)
             log.debug("Trying to get service auth type for %s", service_endpoint)
-            with BaseProtocol.raw_session(service_endpoint) as s:
+            with protocol.raw_session(service_endpoint) as s:
                 try:
                     r = s.post(
                         url=service_endpoint,
                         headers=headers,
                         data=data,
                         allow_redirects=False,
-                        timeout=BaseProtocol.TIMEOUT,
+                        timeout=protocol.TIMEOUT,
                     )
                     r.close()  # Release memory
                     break
@@ -182,12 +119,66 @@ def get_service_authtype(service_endpoint, retry_policy, api_versions, name):
         try:
             auth_type = get_auth_method_from_response(response=r)
             log.debug("Auth type is %s", auth_type)
-            return auth_type, api_version
+            return auth_type
         except UnauthorizedError:
             continue
     raise TransportError("Failed to get auth type from service")
 
 
+def get_autodiscover_authtype(protocol):
+    data = protocol.dummy_xml()
+    headers = {"X-AnchorMailbox": "DUMMY@example.com"}  # Required in case of OAuth
+    r = get_unauthenticated_autodiscover_response(protocol=protocol, method="post", headers=headers, data=data)
+    auth_type = get_auth_method_from_response(response=r)
+    log.debug("Auth type is %s", auth_type)
+    return auth_type
+
+
+def get_unauthenticated_autodiscover_response(protocol, method, headers=None, data=None):
+    from .autodiscover import Autodiscovery
+
+    service_endpoint = protocol.service_endpoint
+    retry_policy = protocol.retry_policy
+    retry = 0
+    t_start = time.monotonic()
+    while True:
+        _back_off_if_needed(retry_policy.back_off_until)
+        log.debug("Trying to get response from %s", service_endpoint)
+        with protocol.raw_session(service_endpoint) as s:
+            try:
+                r = getattr(s, method)(
+                    url=service_endpoint,
+                    headers=headers,
+                    data=data,
+                    allow_redirects=False,
+                    timeout=protocol.TIMEOUT,
+                )
+                r.close()  # Release memory
+                break
+            except TLS_ERRORS as e:
+                # Don't retry on TLS errors. But wrap, so we can catch later and continue with the next endpoint.
+                raise TransportError(str(e))
+            except CONNECTION_ERRORS as e:
+                r = DummyResponse(url=service_endpoint, request_headers=headers)
+                total_wait = time.monotonic() - t_start
+                if retry_policy.may_retry_on_error(response=r, wait=total_wait):
+                    log.debug(
+                        "Connection error on URL %s (retry %s, error: %s). Cool down %s secs",
+                        service_endpoint,
+                        retry,
+                        e,
+                        Autodiscovery.RETRY_WAIT,
+                    )
+                    # Don't respect the 'Retry-After' header. We don't know if this is a useful endpoint, and we
+                    # want autodiscover to be reasonably fast.
+                    retry_policy.back_off(Autodiscovery.RETRY_WAIT)
+                    retry += 1
+                    continue
+                log.debug("Connection error on URL %s: %s", service_endpoint, e)
+                raise TransportError(str(e))
+    return r
+
+
 def get_auth_method_from_response(response):
     # First, get the auth method from headers. Then, test credentials. Don't handle redirects - burden is on caller.
     log.debug("Request headers: %s", response.request.headers)
@@ -211,6 +202,8 @@ def get_auth_method_from_response(response):
                 return NTLM
             if "basic" in vals:
                 return BASIC
+        elif key.lower() == "ms-diagnostics-public" and "Modern Auth" in val:
+            return OAUTH2
     raise UnauthorizedError("No compatible auth type was reported by server")
 
 
@@ -236,19 +229,3 @@ def _tokenize(val):
     if auth_method:
         auth_methods.append(auth_method)
     return auth_methods
-
-
-def dummy_xml(api_version, name):
-    # Generate a minimal, valid EWS request
-    from .services import ResolveNames  # Avoid circular import
-
-    return wrap(
-        content=ResolveNames(protocol=None).get_payload(
-            unresolved_entries=[name],
-            parent_folders=None,
-            return_full_contact_data=False,
-            search_scope=None,
-            contact_data_shape=None,
-        ),
-        api_version=api_version,
-    )
