@@ -25,14 +25,7 @@ from pygments.formatters.terminal import TerminalFormatter
 from pygments.lexers.html import XmlLexer
 from requests_oauthlib import OAuth2Session
 
-from .errors import (
-    InvalidTypeError,
-    MalformedResponseError,
-    RateLimitError,
-    RedirectError,
-    RelativeRedirect,
-    TransportError,
-)
+from .errors import MalformedResponseError, RateLimitError, RedirectError, RelativeRedirect, TransportError
 
 log = logging.getLogger(__name__)
 xml_log = logging.getLogger(f"{__name__}.xml")
@@ -72,14 +65,18 @@ class ElementNotFound(Exception):
         self.data = data
 
 
-# Regex of UTF-8 control characters that are illegal in XML 1.0 (and XML 1.1)
-_ILLEGAL_XML_CHARS_RE = re.compile("[\x00-\x08\x0b\x0c\x0e-\x1F\uD800-\uDFFF\uFFFE\uFFFF]")
+# Regex of UTF-8 control characters that are illegal in XML 1.0 (and XML 1.1).
+# See https://stackoverflow.com/a/22273639/219640
+_ILLEGAL_XML_CHARS_RE = re.compile("[\x00-\x08\x0B-\x0C\x0E-\x1F\x7F-\x84\x86-\x9F\uFDD0-\uFDDF\uFFFE\uFFFF]")
 
 # XML namespaces
 SOAPNS = "http://schemas.xmlsoap.org/soap/envelope/"
 MNS = "http://schemas.microsoft.com/exchange/services/2006/messages"
 TNS = "http://schemas.microsoft.com/exchange/services/2006/types"
 ENS = "http://schemas.microsoft.com/exchange/services/2006/errors"
+ANS = "http://schemas.microsoft.com/exchange/2010/Autodiscover"
+INS = "http://www.w3.org/2001/XMLSchema-instance"
+WSA = "http://www.w3.org/2005/08/addressing"
 AUTODISCOVER_BASE_NS = "http://schemas.microsoft.com/exchange/autodiscover/responseschema/2006"
 AUTODISCOVER_REQUEST_NS = "http://schemas.microsoft.com/exchange/autodiscover/outlook/requestschema/2006"
 AUTODISCOVER_RESPONSE_NS = "http://schemas.microsoft.com/exchange/autodiscover/outlook/responseschema/2006a"
@@ -88,6 +85,9 @@ ns_translation = {
     "s": SOAPNS,
     "m": MNS,
     "t": TNS,
+    "a": ANS,
+    "wsa": WSA,
+    "xsi": INS,
 }
 for item in ns_translation.items():
     lxml.etree.register_namespace(*item)
@@ -247,7 +247,6 @@ def set_xml_value(elem, value, version=None):
     from .ewsdatetime import EWSDate, EWSDateTime
     from .fields import FieldOrder, FieldPath
     from .properties import EWSElement
-    from .version import Version
 
     if isinstance(value, (str, bool, bytes, int, Decimal, datetime.time, EWSDate, EWSDateTime)):
         elem.text = value_to_xml_text(value)
@@ -256,8 +255,6 @@ def set_xml_value(elem, value, version=None):
     elif isinstance(value, (FieldPath, FieldOrder)):
         elem.append(value.to_xml())
     elif isinstance(value, EWSElement):
-        if not isinstance(version, Version):
-            raise InvalidTypeError("version", version, Version)
         elem.append(value.to_xml(version=version))
     elif is_iterable(value, generators_allowed=True):
         for v in value:
@@ -332,7 +329,7 @@ def prepare_input_source(source):
 
 
 def safe_b64decode(data):
-    # Incoming base64-encoded data is not always padded to a multiple of 4. Python's parser is more strict and requires
+    # Incoming base64-encoded data is not always padded to a multiple of 4. Python's parser is stricter and requires
     # padding. Add padding if it's needed.
     overflow = len(data) % 4
     if overflow:
@@ -461,7 +458,7 @@ class DocumentYielder:
 
     def _get_tag(self):
         """Iterate over the bytes until we have a full tag in the buffer. If there's a '>' in an attr value, then we'll
-        exit on that, but it's OK becaus wejust need the plain tag name later.
+        exit on that, but it's OK because we just need the plain tag name later.
         """
         tag_buffer = [b"<"]
         while True:
@@ -553,21 +550,24 @@ def to_xml(bytes_content):
     return res
 
 
-def is_xml(text, expected_prefix=b"<?xml"):
+def is_xml(text):
     """Lightweight test if response is an XML doc. It's better to be fast than correct here.
 
     :param text: The string to check
-    :param expected_prefix: What to search for in the start if the string
     :return:
     """
-    # BOM_UTF8 is an UTF-8 byte order mark which may precede the XML from an Exchange server
+    # BOM_UTF8 is a UTF-8 byte order mark which may precede the XML from an Exchange server
     bom_len = len(BOM_UTF8)
-    prefix_len = len(expected_prefix)
+    expected_prefixes = (b"<?xml", b"<s:Envelope")
+    max_prefix_len = len(expected_prefixes[1])
     if text[:bom_len] == BOM_UTF8:
-        prefix = text[bom_len : bom_len + prefix_len]
+        prefix = text[bom_len : bom_len + max_prefix_len]
     else:
-        prefix = text[:prefix_len]
-    return prefix == expected_prefix
+        prefix = text[:max_prefix_len]
+    for expected_prefix in expected_prefixes:
+        if prefix[: len(expected_prefix)] == expected_prefix:
+            return True
+    return False
 
 
 class PrettyXmlHandler(logging.StreamHandler):
@@ -716,7 +716,6 @@ def get_redirect_url(response, allow_relative=True, require_relative=False):
 
 
 RETRY_WAIT = 10  # Seconds to wait before retry on connection errors
-MAX_REDIRECTS = 10  # Maximum number of URL redirects before we give up
 
 # A collection of error classes we want to handle as general connection errors
 CONNECTION_ERRORS = (
@@ -736,7 +735,7 @@ with suppress(ImportError):
     TLS_ERRORS += (OpenSSL.SSL.Error,)
 
 
-def post_ratelimited(protocol, session, url, headers, data, allow_redirects=False, stream=False, timeout=None):
+def post_ratelimited(protocol, session, url, headers, data, stream=False, timeout=None):
     """There are two error-handling policies implemented here: a fail-fast policy intended for stand-alone scripts which
     fails on all responses except HTTP 200. The other policy is intended for long-running tasks that need to respect
     rate-limiting errors from the server and paper over outages of up to 1 hour.
@@ -766,7 +765,6 @@ def post_ratelimited(protocol, session, url, headers, data, allow_redirects=Fals
     :param url:
     :param headers:
     :param data:
-    :param allow_redirects:  (Default value = False)
     :param stream:  (Default value = False)
     :param timeout:
 
@@ -777,7 +775,6 @@ def post_ratelimited(protocol, session, url, headers, data, allow_redirects=Fals
     thread_id = get_ident()
     wait = RETRY_WAIT  # Initial retry wait. We double the value on each retry
     retry = 0
-    redirects = 0
     log_msg = """\
 Retry: %(retry)s
 Waited: %(wait)s
@@ -787,7 +784,6 @@ Thread: %(thread_id)s
 Auth type: %(auth)s
 URL: %(url)s
 HTTP adapter: %(adapter)s
-Allow redirects: %(allow_redirects)s
 Streaming: %(stream)s
 Response time: %(response_time)s
 Status code: %(status_code)s
@@ -805,7 +801,6 @@ Response XML: %(xml_response)s"""
         auth=session.auth,
         url=url,
         adapter=session.get_adapter(url),
-        allow_redirects=allow_redirects,
         stream=stream,
         response_time=None,
         status_code=None,
@@ -894,7 +889,7 @@ Response XML: %(xml_response)s"""
                 continue
             if r.status_code in (301, 302):
                 r.close()  # Release memory
-                url, redirects = _redirect_or_fail(r, redirects, allow_redirects)
+                url = _fail_on_redirect(r)
                 continue
             break
     except (RateLimitError, RedirectError) as e:
@@ -935,7 +930,7 @@ def _need_new_credentials(response):
     return response.status_code == 401 and response.headers.get("TokenExpiredError")
 
 
-def _redirect_or_fail(response, redirects, allow_redirects):
+def _fail_on_redirect(response):
     # Retry with no delay. If we let requests handle redirects automatically, it would issue a GET to that
     # URL. We still want to POST.
     try:
@@ -943,13 +938,8 @@ def _redirect_or_fail(response, redirects, allow_redirects):
     except RelativeRedirect as e:
         log.debug("'allow_redirects' only supports relative redirects (%s -> %s)", response.url, e.value)
         raise RedirectError(url=e.value)
-    if not allow_redirects:
-        raise TransportError(f"Redirect not allowed but we were redirected ({response.url} -> {redirect_url})")
-    log.debug("HTTP redirected to %s", redirect_url)
-    redirects += 1
-    if redirects > MAX_REDIRECTS:
-        raise TransportError("Max redirect count exceeded")
-    return redirect_url, redirects
+    log.debug("Redirect not allowed but we were redirected ( (%s -> %s)", response.url, redirect_url)
+    raise RedirectError(url=redirect_url)
 
 
 def _retry_after(r, wait):
