@@ -2,66 +2,14 @@ import logging
 import re
 
 from .errors import InvalidTypeError, ResponseMessageError, TransportError
-from .util import TNS, xml_to_str
+from .util import ANS, TNS, get_xml_attr, xml_to_str
 
 log = logging.getLogger(__name__)
 
-# Legend for dict:
-#   Key: shortname
-#   Values: (EWS API version ID, full name)
-
-# 'shortname' comes from types.xsd and is the official version of the server, corresponding to the version numbers
-# supplied in SOAP headers. 'API version' is the version name supplied in the RequestServerVersion element in SOAP
-# headers and describes the EWS API version the server implements. Valid values for this element are described here:
-#    https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/requestserverversion
-
-VERSIONS = {
-    "Exchange2007": ("Exchange2007", "Microsoft Exchange Server 2007"),
-    "Exchange2007_SP1": ("Exchange2007_SP1", "Microsoft Exchange Server 2007 SP1"),
-    "Exchange2007_SP2": ("Exchange2007_SP1", "Microsoft Exchange Server 2007 SP2"),
-    "Exchange2007_SP3": ("Exchange2007_SP1", "Microsoft Exchange Server 2007 SP3"),
-    "Exchange2010": ("Exchange2010", "Microsoft Exchange Server 2010"),
-    "Exchange2010_SP1": ("Exchange2010_SP1", "Microsoft Exchange Server 2010 SP1"),
-    "Exchange2010_SP2": ("Exchange2010_SP2", "Microsoft Exchange Server 2010 SP2"),
-    "Exchange2010_SP3": ("Exchange2010_SP2", "Microsoft Exchange Server 2010 SP3"),
-    "Exchange2013": ("Exchange2013", "Microsoft Exchange Server 2013"),
-    "Exchange2013_SP1": ("Exchange2013_SP1", "Microsoft Exchange Server 2013 SP1"),
-    "Exchange2015": ("Exchange2015", "Microsoft Exchange Server 2015"),
-    "Exchange2015_SP1": ("Exchange2015_SP1", "Microsoft Exchange Server 2015 SP1"),
-    "Exchange2016": ("Exchange2016", "Microsoft Exchange Server 2016"),
-    "Exchange2019": ("Exchange2019", "Microsoft Exchange Server 2019"),
-}
-
-# Build a list of unique API versions, used when guessing API version supported by the server. Use reverse order so we
-# get the newest API version supported by the server.
-API_VERSIONS = sorted({v[0] for v in VERSIONS.values()}, reverse=True)
-
 
 class Build:
     """Holds methods for working with build numbers."""
 
-    # List of build numbers here: https://docs.microsoft.com/en-us/exchange/new-features/build-numbers-and-release-dates
-    API_VERSION_MAP = {
-        8: {
-            0: "Exchange2007",
-            1: "Exchange2007_SP1",
-            2: "Exchange2007_SP1",
-            3: "Exchange2007_SP1",
-        },
-        14: {
-            0: "Exchange2010",
-            1: "Exchange2010_SP1",
-            2: "Exchange2010_SP2",
-            3: "Exchange2010_SP2",
-        },
-        15: {
-            0: "Exchange2013",  # Minor builds starting from 847 are Exchange2013_SP1, see api_version()
-            1: "Exchange2016",
-            2: "Exchange2019",
-            20: "Exchange2016",  # This is Office365. See issue #221
-        },
-    }
-
     __slots__ = "major_version", "minor_version", "major_build", "minor_build"
 
     def __init__(self, major_version, minor_version, major_build=0, minor_build=0):
@@ -92,7 +40,9 @@ class Build:
         for k, xml_elem in xml_elems_map.items():
             v = elem.get(xml_elem)
             if v is None:
-                raise ValueError()
+                v = get_xml_attr(elem, f"{{{ANS}}}{xml_elem}")
+                if v is None:
+                    raise ValueError()
             kwargs[k] = int(v)  # Also raises ValueError
         return cls(**kwargs)
 
@@ -117,15 +67,12 @@ class Build:
         return cls(major_version=major_version, minor_version=minor_version, major_build=build_number)
 
     def api_version(self):
-        if EXCHANGE_2013_SP1 <= self < EXCHANGE_2016:
-            return "Exchange2013_SP1"
-        try:
-            return self.API_VERSION_MAP[self.major_version][self.minor_version]
-        except KeyError:
-            raise ValueError(f"API version for build {self} is unknown")
-
-    def fullname(self):
-        return VERSIONS[self.api_version()][1]
+        for build, api_version, _ in VERSIONS:
+            if self.major_version != build.major_version or self.minor_version != build.minor_version:
+                continue
+            if self >= build:
+                return api_version
+        raise ValueError(f"API version for build {self} is unknown")
 
     def __cmp__(self, other):
         # __cmp__ is not a magic method in Python3. We'll just use it here to implement comparison operators
@@ -173,15 +120,49 @@ class Build:
 # Helpers for comparison operations elsewhere in this package
 EXCHANGE_2007 = Build(8, 0)
 EXCHANGE_2007_SP1 = Build(8, 1)
+EXCHANGE_2007_SP2 = Build(8, 2)
+EXCHANGE_2007_SP3 = Build(8, 3)
 EXCHANGE_2010 = Build(14, 0)
 EXCHANGE_2010_SP1 = Build(14, 1)
 EXCHANGE_2010_SP2 = Build(14, 2)
+EXCHANGE_2010_SP3 = Build(14, 3)
 EXCHANGE_2013 = Build(15, 0)
-EXCHANGE_2013_SP1 = Build(15, 0, 847)
+EXCHANGE_2013_SP1 = Build(15, 0, 847)  # Major builds starting from 847 are Exchange2013_SP1
+EXCHANGE_2015 = Build(15, 20)
+EXCHANGE_2015_SP1 = Build(15, 20)
 EXCHANGE_2016 = Build(15, 1)
 EXCHANGE_2019 = Build(15, 2)
 EXCHANGE_O365 = Build(15, 20)
 
+# Legend for VERSIONS:
+#   (build, API version, full name)
+#
+# 'API version' is the version name supplied in the RequestServerVersion element in SOAP headers and describes the EWS
+# API version the server implements. Valid values for this element are described here:
+#   https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/requestserverversion
+#
+# A list of build numbers and full version names is available here:
+#   https://docs.microsoft.com/en-us/exchange/new-features/build-numbers-and-release-dates
+#
+# The list is sorted from newest to oldest build
+VERSIONS = (
+    (EXCHANGE_O365, "Exchange2016", "Microsoft Exchange Server Office365"),  # Not mentioned in list of build numbers
+    (EXCHANGE_2019, "Exchange2019", "Microsoft Exchange Server 2019"),
+    (EXCHANGE_2016, "Exchange2016", "Microsoft Exchange Server 2016"),
+    (EXCHANGE_2015_SP1, "Exchange2015_SP1", "Microsoft Exchange Server 2015 SP1"),
+    (EXCHANGE_2015, "Exchange2015", "Microsoft Exchange Server 2015"),
+    (EXCHANGE_2013_SP1, "Exchange2013_SP1", "Microsoft Exchange Server 2013 SP1"),
+    (EXCHANGE_2013, "Exchange2013", "Microsoft Exchange Server 2013"),
+    (EXCHANGE_2010_SP3, "Exchange2010_SP2", "Microsoft Exchange Server 2010 SP3"),
+    (EXCHANGE_2010_SP2, "Exchange2010_SP2", "Microsoft Exchange Server 2010 SP2"),
+    (EXCHANGE_2010_SP1, "Exchange2010_SP1", "Microsoft Exchange Server 2010 SP1"),
+    (EXCHANGE_2010, "Exchange2010", "Microsoft Exchange Server 2010"),
+    (EXCHANGE_2007_SP3, "Exchange2007_SP1", "Microsoft Exchange Server 2007 SP3"),
+    (EXCHANGE_2007_SP2, "Exchange2007_SP1", "Microsoft Exchange Server 2007 SP2"),
+    (EXCHANGE_2007_SP1, "Exchange2007_SP1", "Microsoft Exchange Server 2007 SP1"),
+    (EXCHANGE_2007, "Exchange2007", "Microsoft Exchange Server 2007"),
+)
+
 
 class Version:
     """Holds information about the server version."""
@@ -203,7 +184,13 @@ class Version:
 
     @property
     def fullname(self):
-        return VERSIONS[self.api_version][1]
+        for build, api_version, full_name in VERSIONS:
+            if self.build:
+                if self.build.major_version != build.major_version or self.build.minor_version != build.minor_version:
+                    continue
+            if self.api_version == api_version:
+                return full_name
+        raise ValueError(f"Full name for API version {self.api_version} build {self.build} is unknown")
 
     @classmethod
     def guess(cls, protocol, api_version_hint=None):
@@ -217,20 +204,20 @@ class Version:
         :param protocol:
         :param api_version_hint:  (Default value = None)
         """
-        from .services import ResolveNames
+        from .properties import ENTRY_ID, EWS_ID, AlternateId
+        from .services import ConvertId
 
-        # The protocol doesn't have a version yet, so default to latest supported version if we don't have a hint.
-        api_version = api_version_hint or API_VERSIONS[0]
+        # The protocol doesn't have a version yet, so default to the latest supported version if we don't have a hint.
+        api_version = api_version_hint or ConvertId.supported_api_versions()[0]
         log.debug("Asking server for version info using API version %s", api_version)
         # We don't know the build version yet. Hopefully, the server will report it in the SOAP header. Lots of
         # places expect a version to have a build, so this is a bit dangerous, but passing a fake build around is also
-        # dangerous. Make sure the call to ResolveNames does not require a version build.
+        # dangerous.
         protocol.config.version = Version(build=None, api_version=api_version)
-        # Use ResolveNames as a minimal request to the server to test if the version is correct. If not, ResolveNames
-        # will try to guess the version automatically.
-        name = str(protocol.credentials) if protocol.credentials and str(protocol.credentials) else "DUMMY"
+        # Use ConvertId as a minimal request to the server to test if the version is correct. If not, ConvertId will
+        # try to guess the version automatically. Make sure the call to ConvertId does not require a version build.
         try:
-            list(ResolveNames(protocol=protocol).call(unresolved_entries=[name]))
+            list(ConvertId(protocol=protocol).call([AlternateId(id="DUMMY", format=EWS_ID, mailbox="DUMMY")], ENTRY_ID))
         except ResponseMessageError as e:
             # We may have survived long enough to get a new version
             if not protocol.config.version.build:
@@ -248,17 +235,19 @@ class Version:
     def from_soap_header(cls, requested_api_version, header):
         info = header.find(f"{{{TNS}}}ServerVersionInfo")
         if info is None:
-            raise TransportError(f"No ServerVersionInfo in header: {xml_to_str(header)!r}")
+            info = header.find(f"{{{ANS}}}ServerVersionInfo")
+            if info is None:
+                raise TransportError(f"No ServerVersionInfo in header: {xml_to_str(header)!r}")
         try:
             build = Build.from_xml(elem=info)
         except ValueError:
             raise TransportError(f"Bad ServerVersionInfo in response: {xml_to_str(header)!r}")
         # Not all Exchange servers send the Version element
-        api_version_from_server = info.get("Version") or build.api_version()
+        api_version_from_server = info.get("Version") or get_xml_attr(info, f"{{{ANS}}}Version") or build.api_version()
         if api_version_from_server != requested_api_version:
             if cls._is_invalid_version_string(api_version_from_server):
                 # For unknown reasons, Office 365 may respond with an API version strings that is invalid in a request.
-                # Detect these so we can fallback to a valid version string.
+                # Detect these, so we can fall back to a valid version string.
                 log.debug(
                     'API version "%s" worked but server reports version "%s". Using "%s"',
                     requested_api_version,
@@ -279,6 +268,11 @@ class Version:
     def copy(self):
         return self.__class__(build=self.build, api_version=self.api_version)
 
+    @classmethod
+    def all_versions(cls):
+        # Return all supported versions, sorted newest to oldest
+        return [cls(build=build, api_version=api_version) for build, api_version, _ in VERSIONS]
+
     def __eq__(self, other):
         if self.api_version != other.api_version:
             return False
@@ -293,3 +287,51 @@ class Version:
 
     def __str__(self):
         return f"Build={self.build}, API={self.api_version}, Fullname={self.fullname}"
+
+
+class SupportedVersionClassMixIn:
+    """Supports specifying the supported versions of services, fields, folders etc.
+
+    For distinguished folders, a possibly authoritative source is:
+    # https://github.com/OfficeDev/ews-managed-api/blob/master/Enumerations/WellKnownFolderName.cs
+    """
+
+    supported_from = None  # The Exchange build when this element was introduced
+    deprecated_from = None  # The Exchange build when this element was deprecated
+
+    @classmethod
+    def __new__(cls, *args, **kwargs):
+        _check(cls.supported_from, cls.deprecated_from)
+        return super().__new__(cls)
+
+    @classmethod
+    def supports_version(cls, version):
+        return _supports_version(cls.supported_from, cls.deprecated_from, version)
+
+
+class SupportedVersionInstanceMixIn:
+    """Like SupportedVersionClassMixIn but for class instances"""
+
+    def __init__(self, supported_from=None, deprecated_from=None):
+        _check(supported_from, deprecated_from)
+        self.supported_from = supported_from
+        self.deprecated_from = deprecated_from
+
+    def supports_version(self, version):
+        return _supports_version(self.supported_from, self.deprecated_from, version)
+
+
+def _check(supported_from, deprecated_from):
+    if supported_from is not None and not isinstance(supported_from, Build):
+        raise InvalidTypeError("supported_from", supported_from, Build)
+    if deprecated_from is not None and not isinstance(deprecated_from, Build):
+        raise InvalidTypeError("deprecated_from", deprecated_from, Build)
+
+
+def _supports_version(supported_from, deprecated_from, version):
+    # 'version' is a Version instance, for convenience by callers
+    if supported_from and version.build < supported_from:
+        return False
+    if deprecated_from and version.build >= deprecated_from:
+        return False
+    return True
