@@ -4,6 +4,7 @@ import pickle
 import socket
 import tempfile
 import warnings
+from contextlib import suppress
 from unittest.mock import Mock, patch
 
 try:
@@ -17,7 +18,12 @@ from oauthlib.oauth2 import InvalidClientIdError
 
 from exchangelib import close_connections
 from exchangelib.configuration import Configuration
-from exchangelib.credentials import Credentials, OAuth2AuthorizationCodeCredentials, OAuth2Credentials
+from exchangelib.credentials import (
+    BaseOAuth2Credentials,
+    Credentials,
+    OAuth2AuthorizationCodeCredentials,
+    OAuth2Credentials,
+)
 from exchangelib.errors import (
     ErrorAccessDenied,
     ErrorMailRecipientNotFound,
@@ -260,14 +266,12 @@ EWS auth: NTLM""",
         self.assertAlmostEqual(len(list(self.account.protocol.get_timezones())), 130, delta=30, msg=data)
         # Test translation to TimeZone objects
         for tz_definition in self.account.protocol.get_timezones(return_full_timezone_data=True):
-            try:
+            with suppress(ValueError):
                 tz = TimeZone.from_server_timezone(
                     tz_definition=tz_definition,
                     for_year=2018,
                 )
                 self.assertEqual(tz.bias, tz_definition.get_std_and_dst(for_year=2018)[2].bias_in_minutes)
-            except ValueError:
-                pass
 
     def test_get_timezones_parsing(self):
         # Test static XML since it's non-standard
@@ -898,7 +902,7 @@ EWS auth: NTLM""",
         self.assertEqual(ids.count(), len(items))
 
     def test_disable_ssl_verification(self):
-        if isinstance(self.account.protocol.credentials, OAuth2Credentials):
+        if isinstance(self.account.protocol.credentials, BaseOAuth2Credentials):
             self.skipTest("OAuth authentication ony works with SSL verification enabled")
 
         # Test that we can make requests when SSL verification is turned off. I don't know how to mock TLS responses
@@ -996,17 +1000,17 @@ r5p9FrBgavAw5bKO54C0oQKpN/5fta5l6Ws0
     MajorVersion="15" MinorVersion="1" MajorBuildNumber="2345" MinorBuildNumber="6789" Version="V2017_07_11"/>
   </s:Header>
   <s:Body>
-    <m:ResolveNamesResponse
+    <m:ConvertIdResponse
     xmlns:m="http://schemas.microsoft.com/exchange/services/2006/messages"
     xmlns:t="http://schemas.microsoft.com/exchange/services/2006/types">
       <m:ResponseMessages>
-        <m:ResolveNamesResponseMessage ResponseClass="Error">
-          <m:MessageText>Multiple results were found.</m:MessageText>
-          <m:ResponseCode>ErrorNameResolutionMultipleResults</m:ResponseCode>
+        <m:ConvertIdResponseMessage ResponseClass="Error">
+          <m:MessageText>The SMTP address format is invalid.</m:MessageText>
+          <m:ResponseCode>ErrorInvalidSmtpAddress</m:ResponseCode>
           <m:DescriptiveLinkKey>0</m:DescriptiveLinkKey>
-        </m:ResolveNamesResponseMessage>
+        </m:ConvertIdResponseMessage>
       </m:ResponseMessages>
-    </m:ResolveNamesResponse>
+    </m:ConvertIdResponse>
   </s:Body>
 </s:Envelope>""",
         )
@@ -1023,24 +1027,26 @@ r5p9FrBgavAw5bKO54C0oQKpN/5fta5l6Ws0
   <s:Header>
   </s:Header>
   <s:Body>
-    <m:ResolveNamesResponse
+    <m:ConvertIdResponse
     xmlns:m="http://schemas.microsoft.com/exchange/services/2006/messages"
     xmlns:t="http://schemas.microsoft.com/exchange/services/2006/types">
       <m:ResponseMessages>
-        <m:ResolveNamesResponseMessage ResponseClass="Error">
-          <m:MessageText>.</m:MessageText>
-          <m:ResponseCode>ErrorNameResolutionMultipleResults</m:ResponseCode>
+        <m:ConvertIdResponseMessage ResponseClass="Error">
+          <m:MessageText>The SMTP address format is invalid.</m:MessageText>
+          <m:ResponseCode>ErrorInvalidSmtpAddress</m:ResponseCode>
           <m:DescriptiveLinkKey>0</m:DescriptiveLinkKey>
-        </m:ResolveNamesResponseMessage>
+        </m:ConvertIdResponseMessage>
       </m:ResponseMessages>
-    </m:ResolveNamesResponse>
+    </m:ConvertIdResponse>
   </s:Body>
 </s:Envelope>""",
         )
         with self.assertRaises(TransportError) as e:
             Version.guess(protocol)
         self.assertEqual(
-            e.exception.args[0], "No valid version headers found in response (ErrorNameResolutionMultipleResults('.'))"
+            e.exception.args[0],
+            "No valid version headers found in response "
+            "(ErrorInvalidSmtpAddress('The SMTP address format is invalid.'))",
         )
 
     @patch("requests.sessions.Session.post", side_effect=ConnectionResetError("XXX"))
