@@ -1,5 +1,6 @@
 import io
 import logging
+from contextlib import suppress
 from itertools import chain
 from unittest.mock import patch
 
@@ -91,13 +92,13 @@ class UtilTest(EWSTest):
         # map
         is_empty, seq = peek(map(int, []))
         self.assertEqual((is_empty, list(seq)), (True, []))
-        is_empty, seq = peek(map(int, [1, 2, 3]))
+        is_empty, seq = peek(map(int, (1, 2, 3)))
         self.assertEqual((is_empty, list(seq)), (False, [1, 2, 3]))
 
         # generator
-        is_empty, seq = peek((i for i in []))
+        is_empty, seq = peek((i for i in ()))
         self.assertEqual((is_empty, list(seq)), (True, []))
-        is_empty, seq = peek((i for i in [1, 2, 3]))
+        is_empty, seq = peek((i for i in (1, 2, 3)))
         self.assertEqual((is_empty, list(seq)), (False, [1, 2, 3]))
 
     @requests_mock.mock()
@@ -157,6 +158,8 @@ class UtilTest(EWSTest):
     def test_is_xml(self):
         self.assertEqual(is_xml(b'<?xml version="1.0" encoding="UTF-8"?><foo></foo>'), True)
         self.assertEqual(is_xml(BOM_UTF8 + b'<?xml version="1.0" encoding="UTF-8"?><foo></foo>'), True)
+        self.assertEqual(is_xml(b"<s:Envelope><foo></foo></s:Envelope>"), True)
+        self.assertEqual(is_xml(BOM_UTF8 + b"<s:Envelope><foo></foo></s:Envelope>"), True)
         self.assertEqual(is_xml(b"XXX"), False)
 
     def test_xml_to_str(self):
@@ -229,7 +232,6 @@ class UtilTest(EWSTest):
         protocol = self.account.protocol
         orig_policy = protocol.config.retry_policy
         RETRY_WAIT = exchangelib.util.RETRY_WAIT
-        MAX_REDIRECTS = exchangelib.util.MAX_REDIRECTS
 
         session = protocol.get_session()
         try:
@@ -281,13 +283,6 @@ class UtilTest(EWSTest):
             session.post = mock_post(url, 302, {"location": "https://contoso.com"})
             with self.assertRaises(TransportError):
                 r, session = post_ratelimited(protocol=protocol, session=session, url=url, headers=None, data="")
-            # Redirect header to other location and allow_redirects=True
-            exchangelib.util.MAX_REDIRECTS = 0
-            session.post = mock_post(url, 302, {"location": "https://contoso.com"})
-            with self.assertRaises(TransportError):
-                r, session = post_ratelimited(
-                    protocol=protocol, session=session, url=url, headers=None, data="", allow_redirects=True
-                )
 
             # CAS error
             session.post = mock_post(url, 999, {"X-CasErrorCode": "AAARGH!"})
@@ -338,12 +333,9 @@ class UtilTest(EWSTest):
             # Restore patched attributes and functions
             protocol.config.retry_policy = orig_policy
             exchangelib.util.RETRY_WAIT = RETRY_WAIT
-            exchangelib.util.MAX_REDIRECTS = MAX_REDIRECTS
 
-            try:
+            with suppress(AttributeError):
                 delattr(protocol, "renew_session")
-            except AttributeError:
-                pass
 
     def test_safe_b64decode(self):
         # Test correctly padded string
